-- "DnD Text Adventure"
-- By Tim Sandford, 2023
-- Comments to timsandford42@hotmail.com

require("file42")
require("math42")
require("music42")
require("table42")
require("text-to-speech")
require("asl")

function love.run()   -- Main function
  gVERSION, gPATCH, gREVISION, gCODENAME = love.getVersion()
  if gVERSION >=  11 then  -- Colour fix for 11.0 onwards
    local oldSetColor = love.graphics.setColor
    love.graphics.setColor = function (r, g, b, a)     
      if type(r)=="table" then
        r, g, b, a = r[1] / 255, r[2] / 255, r[3] / 255, (r[4] or 255) / 255
      else
        r, g, b, a = r / 255, g / 255, b / 255, (a or 255) / 255
      end
      oldSetColor(r,g,b,a)
    end
  end
  math.randomseed(os.time())  -- True randomness
  love.load(love.arg.parseGameArguments(arg), arg) -- load variables etc.
  while true do  -- Main loop
    if love.event then -- Process events
      love.event.pump()
      for name, a,b,c,d,e,f in love.event.poll() do
        if name == "quit" then
          if not love.quit or not love.quit() then
            return a
          end
        end
        love.handlers[name](a,b,c,d,e,f)
      end
    end
    love.update() -- Update
    love.graphics.origin() -- Not sure, probably reset origin?
    love.graphics.clear(love.graphics.getBackgroundColor()) -- Clears the screen
    love.draw() -- Collect draw commands
    love.graphics.present() -- Present the graphics on the screen!
    love.timer.sleep(0.001) -- This is to lower the load on the CPU, but I could edit it :)	
  end
end

function love.load(arg)  -- Load everything in
  local i, j, list
  io.stdout:setvbuf("no")                                         -- Print messages immediately, allows tracing bugs
  if arg[#arg] == "-debug" then require("mobdebug").start() end   -- Zerobrane debugging
  gSYSTEM = love.system.getOS()                                   -- "OS X", "Windows", "Linux", "Android" or "iOS"
  if (gSYSTEM=="iOS" or gSYSTEM=="Android") then
    gORIENTATION = "portrait"                                       -- Portrait mode on phones
    gSCREEN_H, gSCREEN_W = love.graphics.getDimensions()            -- Get screensize, switched on phone default to landscape
    gCANVAS_W, gCANVAS_H = 300, 400                                 -- 300x400 canvas
    if gVERSION>11 or (gVERSION==11 and gPATCH>=4) then love.window.updateMode(gCANVAS_W, gCANVAS_H) end  -- Portrait from 11.4
    gRATIOX, gRATIOY = gSCREEN_W/gCANVAS_W, gSCREEN_H/gCANVAS_H     -- Scale graphics to screen
  else
    gORIENTATION = "landscape"                                      -- Landscape mode is normal
    gSCREEN_W, gSCREEN_H = love.graphics.getDimensions()            -- Get screensize, desktop normal
    gCANVAS_W, gCANVAS_H = 400, 300                                 -- 400x300 canvas
    gRATIOX, gRATIOY = gSCREEN_W/gCANVAS_W, gSCREEN_H/gCANVAS_H     -- Scale graphics to screen
  end
  gFADE, gFADESPEED = 255, 5                                       -- Fade in effect
  gTIMER = love.timer.getTime()                                    -- Use timer in love.update for speed of game
  gFLASH, gFLASH_z, gFLASH_min, gFLASH_max = 255, 10, 80, 255		  -- Flashing images/text transparency
  gSMOKE, gSMOKE_z, gSMOKE_speed = 1, 0, 3                        -- Swirling smoke effect
  mTRACK, gTRACK, gFADEIN, gFADEOUT = {}, 1, 0, 0                  -- Music tracks
  mTRACK[1], mTRACK[2], mBATTLE, mCOUNT = {}, {}, false, 0         -- Music tracks
  setnottyping()                                                  -- gTYPING, gTYPEDTEXT, gSUGGESTED and setTextInput(false)
  gFLOATTEXT = {}                                                  -- Floating text on screen

  fTINY = love.graphics.newFont("assets/fonts/regular.ttf", 8)            -- tiny font
  fSMALL = love.graphics.newFont("assets/fonts/regular.ttf", 10)          -- small font
  fDEFAULT = love.graphics.newFont("assets/fonts/regular.ttf", 12)        -- default font
  fSUBHEADING = love.graphics.newFont("assets/fonts/heading.ttf", 16)     -- sub heading font
  fHEADING = love.graphics.newFont("assets/fonts/heading.ttf", 20)        -- heading font
  fTITLE = love.graphics.newFont("assets/fonts/heading.ttf", 32)          -- title font
  fHUGE = love.graphics.newFont("assets/fonts/heading.ttf", 64)           -- huge font

  cHAND = love.mouse.newCursor("assets/cursors/hand.png", 0, 0)           -- hand cursor
  cSWORD = love.mouse.newCursor("assets/cursors/sword.png", 0, 0)         -- sword cursor (attack)
  love.mouse.setCursor(cHAND)
  love.keyboard.setTextInput(false)

  iUNKNOWN = love.graphics.newImage("assets/gui/unknown.jpg")             -- unknown image

  initialise()   -- add default adventure etc.

  tSAVE = getordereddirectory("Saves", ".txt")
  tFILENAME = getordereddirectory("Adventures")
  tCHAR = getordereddirectory("Characters")

  sSOUND = {}
  list = getordereddirectory("assets/sounds", ".ogg")
  if next(list)~=nil then
    for i = 1, #list, 1 do
      sSOUND[string.sub(list[i], 1, string.len(list[i])-4)] = love.audio.newSource("assets/sounds/"..list[i], "static")
    end
  end

  iICON = {}
  list = getordereddirectory("assets/gui", ".png")
  if next(list)~=nil then
    for i = 1, #list, 1 do
      iICON[string.sub(list[i], 1, string.len(list[i])-4)] = love.graphics.newImage("assets/gui/"..list[i])
    end
  end

  eSMOKE = {}
  for i = 1, 99, 1 do
    if love.filesystem.getInfo("assets/effects/smoke"..i..".png", "file") then eSMOKE[i] = love.graphics.newImage("assets/effects/smoke"..i..".png") end
  end

  if love.filesystem.getInfo("settings.txt", "file") then tSETTINGS = readsingletable("settings.txt")
  else tSETTINGS = readsingletable("data/defaultsettings.txt") end

  musicload(tSETTINGS.MusicVol)
  setup_mainmenu()
end

function initialise()  -- Add default adventure etc.
  local i, list
  if not love.filesystem.getInfo("Autosave", "directory") then love.filesystem.createDirectory("Autosave") end
  if not love.filesystem.getInfo("Adventures", "directory") then love.filesystem.createDirectory("Adventures") end
  if not love.filesystem.getInfo("cache") then love.filesystem.createDirectory("cache") end
  if not love.filesystem.getInfo("Saves", "directory") then love.filesystem.createDirectory("Saves") end
  if not love.filesystem.getInfo("Characters", "directory") then love.filesystem.createDirectory("Characters") end
  if not love.filesystem.getInfo("Music", "directory") then 
    love.filesystem.createDirectory("Music") 
    list = getordereddirectory("assets/default_music", ".mp3")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        copyfile("assets/default_music/"..list[i], "Music/"..list[i])
      end
    end
  end
  if not love.filesystem.getInfo("Adventures/Bandit Trouble", "directory") and love.filesystem.getInfo("default adventure", "directory") then
    love.filesystem.createDirectory("Adventures/Bandit Trouble")
    list = getordereddirectory("default adventure")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        copyfile("default adventure/"..list[i], "Adventures/Bandit Trouble/"..list[i])
      end
    end
  end
end

function love.update()  -- Runs every frame
  updategeneral()
  if _G["update"..gGAMEMODE]~=nil then _G["update"..gGAMEMODE]() end
  musicupdate()
  collectgarbage()
  if love.timer.getTime()-gTIMER<=(1/tSETTINGS.FPS) then love.timer.sleep((1/tSETTINGS.FPS)-(love.timer.getTime()-gTIMER)) end  -- frame rate
  gTIMER = love.timer.getTime()  -- new timer
end

function updategeneral()  -- Updates general variables, fade and flash
  --if love.mouse.isDown(1) then gMOUSEGRAB = gMOUSEGRAB + 1 end
  if gFADE > 0 then  -- fade in graphics
    gFADE = gFADE - gFADESPEED
    if gFADE < 0 then gFADE = 0 end
  end
  gFLASH = gFLASH + gFLASH_z  -- flash counter
  if gFLASH > 255 then  -- this affects everything that flashes (same speed)
    gFLASH = gFLASH_max
    gFLASH_z = 0 - gFLASH_z
  elseif gFLASH < gFLASH_min then
    gFLASH = gFLASH_min
    gFLASH_z = 0 - gFLASH_z
  else end
  gSMOKE_z = gSMOKE_z + 1
  if gSMOKE_z >= gSMOKE_speed then
    gSMOKE_z = 0
    gSMOKE = gSMOKE + 1
    if gSMOKE > 90 then gSMOKE = 1 end
  end
end

function updategame()   -- Updates when game is running
  local punc = {"%.", "%,", "%?", "%!"}
  local i, j, l, n
  if safetest(tSETTINGS.Speech, "Yes") then
    if gSPEECH_LINE<=gettablesize(tAREA_TEXT) then gSPEECH_STRING = gSPEECH_STRING..tAREA_TEXT[gSPEECH_LINE] end
    n = 0
    repeat
      n = n + 1
      i, j = string.find(gSPEECH_STRING, punc[n])
    until (i~=nil and j~=nil) or n==#punc
    l = string.len(gSPEECH_STRING)
    if i~=nil and j~=nil then
      create_speech(string.sub(gSPEECH_STRING, 1, i))
      if l>i then gSPEECH_STRING = string.sub(gSPEECH_STRING, i+1, l)
      else gSPEECH_STRING = "" end
    end
    gSPEECH_LINE = gSPEECH_LINE + 1
    if love.filesystem.getInfo("cache/line"..(gSPEECH_PLAYING+1)..".mp3") and (gSPEECH_PLAYING==0 or not tSPEECH[gSPEECH_PLAYING]:isPlaying()) then
      gSPEECH_PLAYING = gSPEECH_PLAYING + 1
      tSPEECH[gSPEECH_PLAYING] = love.audio.newSource("cache/line"..gSPEECH_PLAYING..".mp3", "static")
      tSPEECH[gSPEECH_PLAYING]:setTimeStretch(2.0)
      tSPEECH[gSPEECH_PLAYING]:play()
    end
  end
end

function love.draw()  -- Draw everything in, based on gamemode
  if gORIENTATION=="portrait" and (gVERSION<11 or (gVERSION==11 and gPATCH<=3)) then  -- Need to rotate the screen before 11.4
    love.graphics.translate(0, gSCREEN_W)
    love.graphics.rotate(-math.pi/2)
  end
  love.graphics.scale(gRATIOX, gRATIOY)  -- Scale to fit the screen
  if _G["draw"..gGAMEMODE]~=nil then _G["draw"..gGAMEMODE]() end
  if gFADE > 0 then 
    love.graphics.setColor(0, 0, 0, gFADE)
    love.graphics.rectangle("fill", 0, 0, gCANVAS_W, gCANVAS_H)
  else end
end

function drawbuttons()  -- Draws the contents of tBUTTON
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local i
  if tBUTTON~=nil and next(tBUTTON)~=nil then
    for i = 1, #tBUTTON, 1 do
      love.graphics.setColor(255, 255, 255, tBUTTON[i].T)
      if tBUTTON[i].Clickable == "Yes" and mx>tBUTTON[i].X-(tBUTTON[i].W/2) and mx<tBUTTON[i].X+(tBUTTON[i].W/2) and my>tBUTTON[i].Y-(tBUTTON[i].H/2) and my<tBUTTON[i].Y+(tBUTTON[i].H/2) then drawimage(tBUTTON[i].Alt, tBUTTON[i].X, tBUTTON[i].Y, 0, tBUTTON[i].W, tBUTTON[i].H)
      else drawimage(tBUTTON[i].Image, tBUTTON[i].X, tBUTTON[i].Y, 0, tBUTTON[i].W, tBUTTON[i].H) end
      love.graphics.setFont(tBUTTON[i].Font)
      love.graphics.setColor(tBUTTON[i].R, tBUTTON[i].G, tBUTTON[i].B, tBUTTON[i].T)
      love.graphics.printf(tBUTTON[i].Text, tBUTTON[i].X-(tBUTTON[i].W/2), tBUTTON[i].Y-(tBUTTON[i].Font:getHeight()/2), tBUTTON[i].W, tBUTTON[i].Align)
    end
  end
end

function drawborder()  -- Draws the fancy corner border
  love.graphics.setColor(255, 255, 255, 255)
  drawimage(iICON["corner1"], 40, 40, 0, 80, 80)
  drawimage(iICON["corner2"], gCANVAS_W-40, 40, 0, 80, 80)
  drawimage(iICON["corner3"], gCANVAS_W-40, gCANVAS_H-40, 0, 80, 80)
  drawimage(iICON["corner4"], 40, gCANVAS_H-40, 0, 80, 80)
end

function drawimage(image, x, y, r, size_x, size_y)  -- Draw centred image of specific size
  local w, h = image:getWidth(), image:getHeight()
  r, size_x, size_y = r or 0, size_x or w, size_y or h
  love.graphics.draw(image, x, y, r, size_x/w, size_y/h, w/2, h/2) 
end

function drawmenu()  -- Draw the main menu
  drawsmoke()    -- Draws a smokey background
  drawborder()   -- Draws the corner border
  drawbuttons()  -- Draws buttons on the screen
  if gTYPING == true then drawtypedtext() end  -- Draws typed text (e.g. character name)
end

function drawsmoke()  -- Draws a smokey background in the middle of the screen
  love.graphics.setColor(255, 255, 255, 255)
  drawimage(eSMOKE[gSMOKE], gCANVAS_W/2, gCANVAS_H/2, 0, gCANVAS_W*1.5, gCANVAS_H*1.5)
end

function drawtypedtext()   -- Draw typed text
  love.graphics.setFont(fSUBHEADING)
  love.graphics.setColor(255, 255, 255, 255)
  drawimage(iICON["header"], gCANVAS_W*0.5, gCANVAS_H*0.5, 0, iICON["header"]:getWidth(), iICON["header"]:getHeight())
  love.graphics.printf(gTYPEDTEXT, (gCANVAS_W*0.5)-(iICON["header"]:getWidth()*0.45), (gCANVAS_H*0.5)-8, iICON["header"]:getWidth()*0.9, "left")
end

function drawgame()   -- Draw the gamescreen
  drawborder()    -- Draws the corner border
  drawbuttons()   -- Draws the main screen buttons
  if gSUBMENU=="main" then drawareatext() end  -- Draws the area text
  if gSUBMENU=="profile" then
    if gPROFILE_PAGE==2 then drawskillarrows()
    elseif gPROFILE_PAGE==3 then drawinvarrows()
    else end
  end
end

function drawareatext()    -- Draws the description, line by line
  local i, j, max_lines
  love.graphics.setFont(fDEFAULT)
  love.graphics.setColor(255, 255, 255, 255)
  if gORIENTATION=="portrait" then max_lines=15
  else max_lines=10 end
  if next(tAREA_TEXT)~=nil then
    j = 55
    for i = gLINE, math.min(#tAREA_TEXT, gLINE+(max_lines-1)), 1 do
      love.graphics.printf(tAREA_TEXT[i], (gCANVAS_W*0.5)-100, j, 200, "left")
      j = j + 15
    end
  end
  if gORIENTATION=="portrait" then
    if gettablesize(tAREA_TEXT)>15 then
      drawimage(iICON["arrowdown"], (gCANVAS_W*0.5)+110, 275, 0, 17, 17)
      drawimage(iICON["arrowup"], (gCANVAS_W*0.5)+110, 65, 0, 17, 17)
    end
  else
    if gettablesize(tAREA_TEXT)>10 then
      drawimage(iICON["arrowdown"], (gCANVAS_W*0.5)+110, 200, 0, 17, 17)
      drawimage(iICON["arrowup"], (gCANVAS_W*0.5)+110, 65, 0, 17, 17)
    end
  end
  if safetest(tSETTINGS.Speech, "Yes") then drawimage(iICON["audioon"], 30, gCANVAS_H-20, 0, 17, 17)
  else drawimage(iICON["audiooff"], 30, gCANVAS_H-20, 0, 17, 17) end
  drawimage(iICON["refresh"], 30, gCANVAS_H-80, 0, 17, 17)
  drawimage(iICON["profile"], gCANVAS_W-30, gCANVAS_H-20, 0, 17, 17)
end

function drawskillarrows()  -- Draw scroll up/down arrows
  love.graphics.setColor(255, 255, 255, 255)
  drawimage(iICON["arrowup"], (gCANVAS_W*0.5)+91, 66, 0, 17, 17)
  if gORIENTATION == "portrait" then 
  else drawimage(iICON["arrowdown"], (gCANVAS_W*0.5)+91, 192, 0, 17, 17) end
end

function drawinvarrows()  -- Draw scroll up/down arrows
  love.graphics.setColor(255, 255, 255, 255)
  drawimage(iICON["arrowup"], (gCANVAS_W*0.5)+111, 66, 0, 17, 17)
  if gORIENTATION == "portrait" then
  else drawimage(iICON["arrowdown"], (gCANVAS_W*0.5)+111, 192, 0, 17, 17) end
end

function love.mousemoved(x, y, dx, dy, istouch)  -- Moved the mouse
  x,y = x/gRATIOX, y/gRATIOY
  if _G["moved"..gGAMEMODE]~=nil then
    _G["moved"..gGAMEMODE](x, y, dx, dy)
  end
end

function love.mousepressed(x, y, button, istouch, presses)  -- Record any click, based on gamemode
  if gORIENTATION == "portrait" then x,y = (gSCREEN_W-y)/gRATIOX, x/gRATIOY
  else x,y = x/gRATIOX, y/gRATIOY end
  if _G["click"..gGAMEMODE]~=nil then
    _G["click"..gGAMEMODE](x, y, button, istouch, presses)
  end
end

function clickbuttons(x, y, button, istouch, presses)  -- Process click on menu buttons
  local clicked = getbuttonclicked(x, y)
  if clicked ~= 0 then
    playsound("accept")
    if tBUTTON[clicked].Var~=nil and tBUTTON[clicked].Var~="None" then _G[tBUTTON[clicked].Script](tBUTTON[clicked].Var)
    else _G[tBUTTON[clicked].Script]() end
  end
end

function getbuttonclicked(x, y)  -- Which button was clicked at this location?
  local clicked = 0
  local i
  if tBUTTON~=nil and next(tBUTTON)~=nil then
    for i = 1, #tBUTTON, 1 do
      if tBUTTON[i].Clickable == "Yes" and x>tBUTTON[i].X-(tBUTTON[i].W/2) and x<tBUTTON[i].X+(tBUTTON[i].W/2) and y>tBUTTON[i].Y-(tBUTTON[i].H/2) and y<tBUTTON[i].Y+(tBUTTON[i].H/2) and tBUTTON[i].Script~="None" and _G[tBUTTON[i].Script]~=nil then clicked = i end
    end
  end
  return clicked
end

function clickmenu(x, y, button, istouch, presses)  -- Clicked the menu
  clickbuttons(x, y, button, istouch, presses)  -- Mousepressed event, buttons
end

function clickgame(x, y, button, istouch, presses)   -- Clicked during the game
  clickbuttons(x, y, button, istouch, presses)
end

function setnottyping()  -- Set not typing text in
  gTYPING = false
  gTYPEDTEXT, gSUGGESTED = "", ""
  love.keyboard.setTextInput(false)
end

function settyping(suggested, length)  -- Receiving text input
  suggested = suggested or ""
  length = length or 20
  gTYPING = true
  gTYPEDTEXT, gSUGGESTED, gTYPEDLENGTH = suggested, suggested, length
  love.keyboard.setTextInput(true)
end

function love.mousereleased(x, y, button, istouch, presses)  -- Released the mouse
  if gORIENTATION == "portrait" then x,y = (gSCREEN_W-y)/gRATIOX, x/gRATIOY
  else x,y = x/gRATIOX, y/gRATIOY end
  if _G["release"..gGAMEMODE]~=nil then
    _G["release"..gGAMEMODE](x,y)
  end
end

function love.textinput(text)  -- Typed text input
  if gTYPING==true then
    if gTYPEDTEXT == gSUGGESTED and gSUGGESTED~="" then gTYPEDTEXT = "" end
    gTYPEDTEXT = gTYPEDTEXT..text
    gTYPEDTEXT = string.gsub(gTYPEDTEXT, '[\"%c]', '')
    if string.len(gTYPEDTEXT)>gTYPEDLENGTH then gTYPEDTEXT=string.sub(gTYPEDTEXT,1,gTYPEDLENGTH) end
  end
end

function love.keypressed(key)  -- Record any keypress, based on gamemode
  if _G["key"..gGAMEMODE]~=nil then 
    _G["key"..gGAMEMODE](key)
  end
end

function keymenu(key)  -- Key during menus
  if key=="escape" then
    if gSUBMENU=="mainmenu" then quitgame() 
    else
      -- Options to go back with key escape go here
    end
  elseif key=="return" then
    if gTYPING==true and gSUBMENU=="create_name" and string.len(gTYPEDTEXT)>0 then
      tPC.Name = gTYPEDTEXT
      setnottyping()
      create_namebuttons()
    end
  elseif key=="backspace" then
    if gTYPING==true and gSUBMENU=="create_name" and string.len(gTYPEDTEXT)>0 then
      gTYPEDTEXT = string.sub(gTYPEDTEXT, 1, string.len(gTYPEDTEXT)-1)
    end
  else end
end

function gettablesize(t)  -- Get table size
  local count = 0
  if t~=nil then
    for _ in pairs(t) do count = count + 1 end
  end
  return count
end

function playsound(i, interrupt)  -- Play sound
  interrupt = interrupt or "No"
  if sSOUND[i]~=nil and tSETTINGS.Sound=="Yes" then 
    if interrupt == "Yes" then 
      sSOUND[i]:stop()
      sSOUND[i]:play()
    else
      if not sSOUND[i]:isPlaying() then sSOUND[i]:play() end
    end
  end
end

function addfloatytext(text, c, interrupt)  -- Add floaty text
  c = c or tSETTINGS.FPS*6
  interrupt = interrupt or "No"
  local entry = {Text=text, Count=c}
  if interrupt == "Yes" then table.insert(gFLOATTEXT, 1, deepcopy(entry))
  else table.insert(gFLOATTEXT, deepcopy(entry)) end
end

function createmenu(title, layout)  -- creates a standard menu with specified button layout
  local width, height, spacing = 100, (gCANVAS_H-80)/10, (gCANVAS_H-80)/9
  local oy, y = height+10, height+10
  local n, z, i, j, columns, rows
  createbutton(title, gCANVAS_W*0.5, 30, {g=215, b=0})
  if layout~=nil and next(layout)~=nil then
    for n = 1, #layout, 1 do
      oy = y
      if next(layout[n])~=nil then
        if (#layout[n] % 2 == 0) then columns = 2
        else columns = 1 end
        rows = #layout[n]/columns
        i, j = 0, 1
        if #layout[n]>0 then
          for z = 1, #layout[n], 1 do
            i = i + 1
            if i>rows then i, j, y = 1, j + 1, oy end
            y = y + spacing
            if layout[n][z].Text~=nil then
              if columns==1 then createbutton(layout[n][z].Text, gCANVAS_W*0.5, y, layout[n][z].Options)
              else createbutton(layout[n][z].Text, (gCANVAS_W*0.5)+((2*(j-1.5))*((width/2)+10)), y, layout[n][z].Options) end
            end
          end
        else 
          i = i + 1
          if i>rows then i, j, y = 1, j + 1, oy end
          y = y + spacing
        end
      else y = y + spacing end
    end
  end
end

function createbutton(text, x, y, option)  -- creates a button, default is header
  local width, wrappedtext, sx, sy
  local z
  option = option or {}
  option.image = option.image or iICON["header"]
  option.width, option.height = option.width or option.image:getWidth(), option.height or option.image:getHeight()
  option.font, option.align = option.font or fHEADING, option.align or "center"
  option.r, option.g, option.b, option.t = option.r or 255, option.g or 255, option.b or 255, option.t or 255
  option.clickable = option.clickable or "No"
  option.alt_image = option.alt_image or option.image
  option.script = option.script or "None"
  option.var = option.var or "None"
  font = font or fDEFAULT
  if tBUTTON==nil then tBUTTON = {} end
  z = gettablesize(tBUTTON)+1
  sx, sy = option.width/option.image:getWidth(), option.height/option.image:getHeight()
  repeat 
    width, wrappedtext = option.font:getWrap(text, option.width)
    if #wrappedtext>1 then
      if option.font==fHUGE then option.font=fTITLE
      elseif option.font==fTITLE then option.font=fHEADING
      elseif option.font==fHEADING then option.font=fSUBHEADING
      elseif option.font==fSUBHEADING then option.font=fDEFAULT
      elseif option.font==fDEFAULT then option.font=fSMALL
      elseif option.font==fSMALL then option.font=fTINY
      else end
    end
  until #wrappedtext == 1 or option.font == fTINY
  tBUTTON[z] = {}
  tBUTTON[z].Text = text
  tBUTTON[z].X, tBUTTON[z].Y = x, y
  tBUTTON[z].Image, tBUTTON[z].Alt = option.image, option.alt_image
  tBUTTON[z].Font, tBUTTON[z].Align = option.font, option.align
  tBUTTON[z].R, tBUTTON[z].G, tBUTTON[z].B, tBUTTON[z].T = option.r, option.g, option.b, option.t
  tBUTTON[z].Clickable = option.clickable
  tBUTTON[z].W, tBUTTON[z].H = option.width, option.height
  tBUTTON[z].Script, tBUTTON[z].Var = option.script, option.var
end

function setup_mainmenu()  -- setup the main menu
  local text = {"New Game", "Load Game", "Options", "Quit"}
  local script = {"click_newgame", "click_loadgame", "click_options", "quitgame"}
  local buttons = {}
  local i
  gGAMEMODE = "menu"
  gSUBMENU = "mainmenu"
  tBUTTON = {}
  if love.filesystem.getInfo("Autosave/Character", "directory") then 
    table.insert(text, 1, "Continue")
    table.insert(script, 1, "loadgame")
  end
  for i = 1, #text, 1 do
    table.insert(buttons, {Text=text[i], Options={image=iICON["button_off"], width=100, font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script=script[i]}})
  end
  if #buttons%2==0 then table.insert(buttons, {}) end
  createmenu("Main Menu", {buttons})
  musicplay(1, "Yes")
end

function click_newgame()  -- clicked new game, now choose which adventure...
  local filename, buttons, accept = {{}}, {{}}, {{}}
  local back = {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="setup_mainmenu"}}}
  gSUBMENU = "newgame_choosegame"
  tADVENTURE = {}
  tBUTTON = {}
  gPAGE = 1
  if next(tFILENAME)~=nil then
    filename = {{Text=tFILENAME[1], Options={font=fSUBHEADING}}}
    accept = {{Text="Accept", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_acceptgame"}}}
    if #tFILENAME>1 then
      buttons = {{Text="<<", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_adventuredown"}}, {Text=">>", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_adventureup"}}}
    end
  end
  createmenu("Choose Adventure", {filename, buttons, accept, back})
end

function click_acceptgame()  -- clicked accept this adventure name, now choose a character...
  local char, buttons, accept = {{}}, {{}}, {{}}
  local new = {{Text="New", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_newchar"}}}
  local back = {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_newgame"}}}
  gSUBMENU = "newgame_choosechar"
  tADVENTURE.Title = tFILENAME[gPAGE]
  tBUTTON = {}
  gCHAR, gPAGE = 0, 1
  if next(tCHAR)~=nil then
    char = {{Text=tCHAR[1]}}
    if #tCHAR>1 then
      buttons = {{Text="<<", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_chardown"}}, {Text=">>", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_charup"}}}
    end
    accept = {{Text="Accept", Options={image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_acceptchar"}}}
  end
  createmenu("Choose a Character", {char, buttons, accept, new, back})
end

function click_acceptchar()  -- Load character then game
  loadcharacter("Characters/"..tCHAR[gPAGE])
  loadgame("Adventures/"..tADVENTURE.Title)
  savegame()
end

function click_adventureup()  -- move up to next file
  gPAGE = gPAGE + 1
  if gPAGE > #tFILENAME then gPAGE = 1 end
  tBUTTON[2].Text = tFILENAME[gPAGE]
end

function click_adventuredown()  -- move down to previous file
  gPAGE = gPAGE - 1
  if gPAGE < 1 then gPAGE = #tFILENAME end
  tBUTTON[2].Text = tFILENAME[gPAGE]
end

function click_newchar()  -- new character, male or female...?
  gSUBMENU = "create_sex"
  tPC = {}
  tPC.Str, tPC.Dex, tPC.Con, tPC.Int, tPC.Wis, tPC.Cha = nil, nil, nil, nil, nil, nil
  tPC.XP = 0
  tPC.Level = 1
  tBUTTON = {}
  createmenu("New Character", {{{Text="Female", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_sex", var="Female"}}, {Text="Male", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_sex", var="Male"}}}, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_acceptgame"}}}})
end

function click_sex(sex)  -- Choose male/female, setup race menu
  tPC.Sex = sex
  setup_racemenu()
end

function setup_racemenu()  -- setup the race menu
  local race = readsinglelist("data/races.txt")
  local buttons = {}
  local i, str
  gSUBMENU = "create_race"
  tPC.Subrace = nil
  tPC.Str_RacBonus, tPC.Dex_RacBonus, tPC.Con_RacBonus = 0, 0, 0
  tPC.Int_RacBonus, tPC.Wis_RacBonus, tPC.Cha_RacBonus = 0, 0, 0
  tPC.BreathWeapon = "No"
  tPC.BreathWeapon_Shape = nil
  tPC.BreathWeapon_Range = nil
  tPC.BreathWeapon_SaveType = nil
  tPC.BreathWeapon_DamageAmount = nil
  tPC.BreathWeapon_DamageType = nil
  tPC.Race_Feat = {}
  tPC.Race_Armor_Proficiency = {}
  tPC.Race_Weapon_Proficiency = {}
  tPC.Race_Tool_Proficiency = {}
  tPC.Race_Skill_Proficiency = {}
  tPC.SpellKnown = {}
  tPC.Race_Language = {}
  gRACE_WIZARDCANTRIP = 0
  gRACE_LANGUAGE = 0
  gPICK_SKILL = 0
  tBUTTON = {}
  for i = 1, #race, 1 do
    str = string.gsub(string.lower(race[i]), "-", "")
    table.insert(buttons, {Text=race[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_"..str}})
  end
  table.insert(buttons, {{}})
  createmenu("Choose a Race", {buttons, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_newchar"}}}})
end

function click_dragonborn()  -- clicked dragonborn, which ancenstry?
  local dragons = {"Black", "Blue", "Brass", "Bronze", "Copper", "Gold", "Green", "Red", "Silver", "White"}
  local buttons = {}
  local k = 1
  local i, j
  gSUBMENU = "create_ancestry"
  tPC.Race = "Dragonborn"
  tPC.Size = "Medium"
  tPC.HeightMin = 72
  tPC.HeightMax = 84
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 230
  tPC.WeightMax = 270
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 30
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Draconic"] = "Yes"
  tPC.Str_RacBonus = 2
  tPC.Cha_RacBonus = 1
  tPC.AgeMin = 15
  tPC.AgeMax = 80
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tBUTTON = {}
  for i = 1, #dragons, 1 do
    table.insert(buttons, {Text=dragons[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_ancestry", var=dragons[i]}})
  end
  createmenu("Choose Draconic Ancestry", {buttons})
end

function click_ancestry(dragon)  -- draconic ancenstry
  local ancestry = {"Black", "Blue", "Brass", "Bronze", "Copper", "Gold", "Green", "Red", "Silver", "White"}
  local shape = {"Line", "Line", "Line", "Line", "Line", "Cone", "Cone", "Cone", "Cone", "Cone"}
  local range = {30, 30, 30, 30, 30, 15, 15, 15, 15, 15}
  local save = {"DEX", "DEX", "DEX", "DEX", "DEX", "DEX", "CON", "DEX", "CON", "CON"}
  local dmgtype = {"Acid", "Lightning", "Fire", "Lightning", "Acid", "Fire", "Poison", "Fire", "Cold", "Cold"}
  local i
  tPC.Draconic_Ancestry = dragon
  for i = 1, #ancestry, 1 do
    if dragon == ancestry[i] then create_breathweapon(shape[i], range[i], save[i], "2d6", dmgtype[i]) end
  end
  setup_classmenu()
end

function create_breathweapon(shape, range, savetype, damageamount, damagetype)  -- create draconic breathweapon
  tPC.BreathWeapon = "Yes"
  tPC.BreathWeapon_Shape = shape
  tPC.BreathWeapon_Range = range
  tPC.BreathWeapon_SaveType = savetype
  tPC.BreathWeapon_DamageAmount = damageamount
  tPC.BreathWeapon_DamageType = damagetype
end

function click_dwarf()  -- clicked dwarf, choose tool proficiency
  tPC.Race = "Dwarf"
  tPC.Size = "Medium"
  tPC.HeightMin = 48
  tPC.HeightMax = 60
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 230
  tPC.WeightMax = 270
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 25
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Dwarvish"] = "Yes"
  tPC.Con_RacBonus = 2
  tPC.AgeMin = 50
  tPC.AgeMax = 350
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Race_Feat["Darkvision"] = 60
  tPC.Race_Feat["Dwarven Resilience"] = "Yes"
  tPC.Race_Feat["Stonecunning"] = "Yes"
  tPC.Race_Weapon_Proficiency["Battleaxe"] = "Yes"
  tPC.Race_Weapon_Proficiency["Handaxe"] = "Yes"
  tPC.Race_Weapon_Proficiency["Light Hammer"] = "Yes"
  tPC.Race_Weapon_Proficiency["Warhammer"] = "Yes"
  setup_dwarftool()
end

function setup_dwarftool()  -- dwarf tool options
  gSUBMENU = "create_dwarftool"
  tBUTTON = {}
  createmenu("Choose Tool Proficiency", {{{Text="Smith Tools", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_dwarftool", var="Smith Tools"}}}, {{Text="Brewer Supplies", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_dwarftool", var="Brewer Supplies"}}}, {{Text="Mason Tools", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_dwarftool", var="Mason Tools"}}}, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_racemenu"}}}})
end

function click_dwarftool(tool)  -- Add Dwarven tool proficiency, setup subrace
  tPC.Race_Tool_Proficiency[tool] = "Yes"
  setup_dwarfsubrace()
end

function setup_dwarfsubrace()  -- Choose Hill Dwarf or Mountain Dwarf
  gSUBMENU = "create_dwarfsubrace"
  tBUTTON = {}
  createmenu("Choose a Subrace", {{{Text="Hill Dwarf", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_hilldwarf"}}}, {{Text="Mount. Dwarf", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_mountdwarf"}}}, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_racemenu"}}}})
end

function click_hilldwarf()  -- Hill dwarf, now choose class
  tPC.Subrace = "Hill Dwarf"
  tPC.Wis_RacBonus = 1
  tPC.Race_Feat["Dwarven Toughness"] = "Yes"
  setup_classmenu()
end

function click_mountdwarf()  -- mountain dwarf, now choose class
  tPC.Subrace = "Mountain Dwarf"
  tPC.Str_RacBonus = 2
  tPC.Race_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Race_Armor_Proficiency["Medium Armor"] = "Yes"
  setup_classmenu()
end

function click_elf()  -- clicked elf, choose subrace
  tPC.Race = "Elf"
  tPC.Size = "Medium"
  tPC.HeightMin = 54
  tPC.HeightMax = 74
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 100
  tPC.WeightMax = 150
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 30
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Elvish"] = "Yes"
  tPC.Dex_RacBonus = 2
  tPC.AgeMin = 50
  tPC.AgeMax = 750
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Race_Feat["Darkvision"] = 60
  tPC.Race_Feat["Fey Ancestry"] = "Yes"
  tPC.Race_Skill_Proficiency["Perception"] = "Yes"
  tPC.Race_Weapon_Proficiency["Longsword"] = "Yes"
  tPC.Race_Weapon_Proficiency["Shortsword"] = "Yes"
  tPC.Race_Weapon_Proficiency["Shortbow"] = "Yes"
  tPC.Race_Weapon_Proficiency["Longbow"] = "Yes"
  setup_elfsubrace()
end

function setup_elfsubrace()  -- Choose elf subrace
  gSUBMENU = "create_elfsubrace"
  tBUTTON = {}
  createmenu("Choose a Subrace", {{{Text="High Elf", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_highelf"}}}, {{Text="Wood Elf", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_woodelf"}}}, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_racemenu"}}}})
end

function click_highelf()  -- high elf, choose class
  tPC.Int_RacBonus = 1
  gRACE_WIZARDCANTRIP = 1
  gRACE_LANGUAGE = 1
  setup_classmenu()
end

function click_woodelf()  -- wood elf, choose class
  tPC.Wis_RacBonus = 1
  tPC.Speed = 35
  tPC.Race_Feat["Mask of the Wild"] = "Yes"
  setup_classmenu()
end

function click_gnome()  -- clicked gnome, choose subrace
  tPC.Int_RacBonus = 2
  tPC.Race = "Gnome"
  tPC.Size = "Small"
  tPC.HeightMin = 36
  tPC.HeightMax = 48
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 30
  tPC.WeightMax = 50
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 25
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Gnomish"] = "Yes"
  tPC.AgeMin = 50
  tPC.AgeMax = 750
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Race_Feat["Darkvision"] = 60
  tPC.Race_Feat["Gnome Cunning"] = "Yes"
  setup_gnomesubrace()
end

function setup_gnomesubrace()  -- setup gnome subraces
  gSUBMENU = "create_gnomesubrace"
  tBUTTON = {}
  createmenu("Choose a Subrace", {{{Text="Forest Gnome", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_forestgnome"}}}, {{Text="Rock Gnome", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_rockgnome"}}}, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_racemenu"}}}})
end

function click_forestgnome()  -- forest gnome, choose class
  tPC.Dex_RacBonus = 1
  tPC.SpellKnown["Minor Illusion"] = "Yes"
  tPC.Race_Feat["Speak with Small Beasts"] = "Yes"
  setup_classmenu()
end

function click_rockgnome()  -- rock gnome, choose class
  tPC.Con_RacBonus = 1
  tPC.Race_Feat["Artificers Lore"] = "Yes"
  tPC.Race_Feat["Tinker"] = "Yes"
  tPC.Race_Tool_Proficiency["Artisan Tools"] = "Yes"
  setup_classmenu()
end

function click_halfelf()  -- half elf, choose 1st ability increase
  local list = {"Strength", "Dexterity", "Constitution", "Intelligence", "Wisdom"}
  local buttons = {}
  local i
  tPC.Race = "Half-Elf"
  tPC.Cha_RacBonus = 2
  tPC.Size = "Medium"
  tPC.HeightMin = 60
  tPC.HeightMax = 74
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 130
  tPC.WeightMax = 190
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 30
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Elvish"] = "Yes"
  gRACE_LANGUAGE = 1
  gPICK_SKILL = 2
  tPC.AgeMin = 20
  tPC.AgeMax = 180
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Race_Feat["Darkvision"] = 60
  tPC.Race_Feat["Fey Ancestry"] = "Yes"
  tBUTTON = {}
  for i = 1, #list, 1 do
    table.insert(buttons, {Text=list[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_halfelf1", var=string.sub(list[i],1,3)}})
  end
  createmenu("1st Ability Increase +1", {buttons})
end

function click_halfelf1(ability)  -- click 1st half-elf ability button
  tPC[ability.."_RacBonus"] = 1
  setup_halfelfbuttons2()
end

function setup_halfelfbuttons2()  -- setup 2nd ability increase buttons
  local list = {"Strength", "Dexterity", "Constitution", "Intelligence", "Wisdom"}
  local buttons = {}
  local i
  tBUTTON = {}
  for i = 1, #list, 1 do
    if tPC[string.sub(list[i],1,3).."_RacBonus"]==0 then
      table.insert(buttons, {Text=list[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_halfelf2", var=string.sub(list[i],1,3)}})
    else
      table.insert(buttons, {Text=list[i], Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
    end
  end
  createmenu("2nd Ability Increase +1", {buttons})
end

function click_halfelf2(ability)  -- 2nd half-elf ability, setup class menu
  tPC[ability.."_RacBonus"] = 1
  setup_classmenu()
end

function click_halforc()  -- clicked half-orc, choose class
  tPC.Str_RacBonus = 2
  tPC.Con_RacBonus = 1
  tPC.Size = "Medium"
  tPC.HeightMin = 60
  tPC.HeightMax = 82
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 150
  tPC.WeightMax = 250
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 30
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Orc"] = "Yes"
  tPC.Race_Skill_Proficiency["Intimidation"] = "Yes"
  tPC.AgeMin = 14
  tPC.AgeMax = 74
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Race_Feat["Darkvision"] = 60
  tPC.Race_Feat["Relentless Endurance"] = "Yes"
  tPC.Race_Feat["Savage Attacks"] = "Yes"
  setup_classmenu()
end

function click_halfling()  -- clicked halfling, choose subrace
  tPC.Race = "Halfling"
  tPC.Dex_RacBonus = 2
  tPC.AgeMin = 20
  tPC.AgeMax = 140
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Size = "Small"
  tPC.HeightMin = 24
  tPC.HeightMax = 48
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.WeightMin = 20
  tPC.WeightMax = 40
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.Speed = 25
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Halfling"] = "Yes"
  tPC.Race_Feat["Lucky"] = "Yes"
  tPC.Race_Feat["Brave"] = "Yes"
  tPC.Race_Feat["Halfling Nimbleness"] = "Yes"
  setup_halflingsubrace()
end

function setup_halflingsubrace()  -- setup halfling subrace buttons
  gSUBMENU = "create_halflingsubrace"
  tBUTTON = {}
  createmenu("Choose a Subrace", {{{Text="Lightfoot", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_lightfoot"}}}, {{Text="Stout", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_stout"}}}, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_racemenu"}}}})
end

function click_lightfoot()  -- lightfoot, choose class
  tPC.Cha_RacBonus = 1
  tPC.Race_Feat["Naturally Stealthy"] = "Yes"
  setup_classmenu()
end

function click_stout()  -- stout, choose class
  tPC.Con_RacBonus = 1
  tPC.Race_Feat["Stout Resilience"] = "Yes"
  setup_classmenu()
end

function click_human()  -- human, choose class
  tPC.Race = "Human"
  tPC.Str_RacBonus = 1
  tPC.Dex_RacBonus = 1
  tPC.Con_RacBonus = 1
  tPC.Int_RacBonus = 1
  tPC.Wis_RacBonus = 1
  tPC.Cha_RacBonus = 1
  tPC.Size = "Medium"
  tPC.HeightMin = 60
  tPC.HeightMax = 78
  tPC.Height = math.floor((tPC.HeightMin+tPC.HeightMax)/2)
  tPC.Speed = 30
  tPC.WeightMin = 140
  tPC.WeightMax = 200
  tPC.Weight = math.floor((tPC.WeightMin+tPC.WeightMax)/2)
  tPC.AgeMin = 18
  tPC.AgeMax = 102
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Race_Language["Common"] = "Yes"
  gRACE_LANGUAGE = 1
  setup_classmenu()
end

function click_tiefling()  -- tiefling, choose class
  tPC.Race = "Tiefling"
  tPC.Race = "Tiefling"
  tPC.Int_RacBonus = 1
  tPC.Cha_RacBonus = 2
  tPC.AgeMin = 18
  tPC.AgeMax = 108
  tPC.Age = math.floor((tPC.AgeMin+tPC.AgeMin+tPC.AgeMax)/3)
  tPC.Size = "Medium"
  tPC.Speed = 30
  tPC.Race_Feat["Darkvision"] = 60
  tPC.Race_Feat["Infernal Legacy"] = "Yes"
  tPC.Race_Feat["Hellish Resistance"] = "Yes"
  tPC.SpellKnown["Thaumaturgy"] = "Yes"
  tPC.Race_Language["Common"] = "Yes"
  tPC.Race_Language["Infernal"] = "Yes"
  setup_classmenu()
end

function setup_classmenu()  -- setup class menu
  local class = readsinglelist("data/classes.txt")
  local buttons = {}
  local i
  gSUBMENU = "create_class"
  tPC.Class = nil
  tPC.Class_Weapon_Proficiency = {}
  tPC.Class_Armor_Proficiency = {}
  tPC.Class_Tool_Proficiency = {}
  tPC.Class_Tool_Expertise = {}
  tPC.Class_Skill_Proficiency = {}
  tPC.Class_Skill_Expertise = {}
  tPC.Saving_Throw_Proficiency = {}
  tPC.Class_Equipment = {}
  tPC.Class_Feat = {}
  gCLASS_SKILLS = 0
  gCLASS_TOOLS = 0
  gDOMAIN_SKILLS = 0
  gCLASS_LANGUAGE = 0
  tPC.Class_Language = {}
  tSPELLS_TO_PICK = {}
  tSPELL_SLOTS = {0, 0, 0, 0, 0, 0, 0, 0, 0}
  tPC.Cleric_Domain = nil
  tPC.Sorcerous_Origin = nil
  tPC.Warlock_Patron = nil
  tBUTTON = {}
  for i = 1, #class, 1 do
    table.insert(buttons, {Text=class[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_"..string.lower(class[i])}})
  end
  createmenu("Choose a Class", {buttons, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_racemenu"}}}})
end

function click_barbarian()  -- barbarian, choose two skills
  tPC.Class = "Barbarian"
  tPC.HitDice = "1d12"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Medium Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Shield"] = "Yes"
  tPC.Saving_Throw_Proficiency["Strength"] = "Yes"
  tPC.Saving_Throw_Proficiency["Constitution"] = "Yes"
  tPC.Class_Equipment = {{"Greataxe", "Greatsword", "Maul"}, {"Two Handaxes", "Light Crossbow", "Spear"}, {"Explorer Pack"}, {"Four Javelins"}}
  simpleweaponprof()
  martialweaponprof()
  gCLASS_SKILLS = 2
  setup_class_skills()
end

function isproficient(creature, item)  -- returns true if character is proficient at skill/ability/weapon/armor/instrument
  local prof = false
  if next(creature)~=nil then
    if creature.Race_Skill_Proficiency~=nil and safetest(creature.Race_Skill_Proficiency[item], "Yes") then prof = true end
    if creature.Class_Skill_Proficiency~=nil and safetest(creature.Class_Skill_Proficiency[item], "Yes") then prof = true end
    if creature.Saving_Throw_Proficiency~=nil and safetest(creature.Saving_Throw_Proficiency[item], "Yes") then prof = true end
    if creature.Race_Armor_Proficiency~=nil and safetest(creature.Race_Armor_Proficiency[item], "Yes") then prof = true end
    if creature.Class_Armor_Proficiency~=nil and safetest(creature.Class_Armor_Proficiency[item], "Yes") then prof = true end
    if creature.Race_Weapon_Proficiency~=nil and safetest(creature.Race_Weapon_Proficiency[item], "Yes") then prof = true end
    if creature.Class_Weapon_Proficiency~=nil and safetest(creature.Class_Weapon_Proficiency[item], "Yes") then prof = true end
    if creature.Race_Tool_Proficiency~=nil and safetest(creature.Race_Tool_Proficiency[item], "Yes") then prof = true end
    if creature.Class_Tool_Proficiency~=nil and safetest(creature.Class_Tool_Proficiency[item], "Yes") then prof = true end
    if creature.Background_Tool_Proficiency~=nil and safetest(creature.Background_Tool_Proficiency[item], "Yes") then prof = true end
    if creature.Race_Skill_Proficiency~=nil and safetest(creature.Race_Skill_Proficiency[item], "Yes") then prof = true end
    if creature.Class_Skill_Proficiency~=nil and safetest(creature.Class_Skill_Proficiency[item], "Yes") then prof = true end
    if creature.Background_Skill_Proficiency~=nil and safetest(creature.Background_Skill_Proficiency[item], "Yes") then prof = true end
  end
  return prof
end

function hasfeat(creature, item)    -- returns true if character has specific feat
  local prof = false
  if next(creature)~=nil then
    if creature.Race_Feat~=nil and safetest(creature.Race_Feat[item], "Yes") then prof = true end
    if creature.Class_Feat~=nil and safetest(creature.Class_Feat[item], "Yes") then prof = true end
  end
  return prof
end

function safetest(var, value)  -- safely compare these two values
  if var == nil or var ~= value then return false
  else return true end
end

function simpleweaponprof()  -- simple weapon proficiency
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  if next(item)~=nil then
    for i = 1, #item, 1 do
      if safetest(item[i].Kind, "Weapon") and safetest(item[i].Simple, "Yes") then
        tPC.Class_Weapon_Proficiency[item[i].Name] = "Yes"
      end
    end
  end
end

function martialweaponprof()  -- martial weapon proficiency
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  if next(item)~=nil then
    for i = 1, #item, 1 do
      if safetest(item[i].Kind, "Weapon") and safetest(item[i].Martial, "Yes") then
        tPC.Class_Weapon_Proficiency[item[i].Name] = "Yes"
      end
    end
  end
end

function click_bard()   -- bard, choose three musical instrument proficiencies
  tPC.Class = "Bard"
  tPC.HitDice = "1d8"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Saving_Throw_Proficiency["Dexterity"] = "Yes"
  tPC.Saving_Throw_Proficiency["Charisma"] = "Yes"
  simpleweaponprof()
  tPC.Class_Weapon_Proficiency["Hand Crossbow"] = "Yes"
  tPC.Class_Weapon_Proficiency["Longsword"] = "Yes"
  tPC.Class_Weapon_Proficiency["Rapier"] = "Yes"
  tPC.Class_Weapon_Proficiency["Shortsword"] = "Yes"
  tPC.Class_Feat["Bardic Inspiration"] = "Yes"
  tPC.Class_Equipment = {{"Rapier", "Longsword", "Light Crossbow"}, {"Diplomat Pack", "Entertainer Pack"}, {"Lute", "Viol", "Bagpipes"}, {"Leather Armor"}, {"Dagger"}}
  tSPELLS_TO_PICK = {"Bard0", "Bard0", "Bard1", "Bard1", "Bard1", "Bard1"}
  tSPELL_SLOTS[1] = 2
  gCLASS_SKILLS = 3
  gCLASS_TOOLS = 3
  setup_musicbuttons()
end

function setup_musicbuttons()  -- instruments available to play
  local buttons, title = {}, ""
  local num = 0
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  gSUBMENU = "create_bardinstrument"
  tBUTTON = {}
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Instrument") and tPC.Class_Tool_Proficiency[item[i].Name]=="Yes" then num = num + 1 end
  end
  if num<gCLASS_TOOLS-1 then title = "Choose "..tostring(gCLASS_TOOLS-num).." Proficiencies"
  else title = "Choose 1 Proficiency" end
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Instrument") then
      if isproficient(tPC, item[i].Name) then table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
      else table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_instrument", var=item[i].Name}}) end
    end
  end
  createmenu(title, {buttons})
end

function click_instrument(name)   -- clicked a bard instrument, proficiency
  local num = 0
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  tPC.Class_Tool_Proficiency[name] = "Yes"
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Instrument") and tPC.Class_Tool_Proficiency[item[i].Name]=="Yes" then num = num + 1 end
  end
  if num < gCLASS_TOOLS then setup_musicbuttons()
  else setup_class_skills() end
end

function setup_class_skills()  -- Setup the class skills that are available
  local skill = readmultitable("data/skills.txt")
  local buttons, available = {}, {}
  local title = ""
  local num = 0
  local i
  gSUBMENU = "create_skills"
  tBUTTON = {}
  for i = 1, #skill, 1 do
    available[i] = skill[i][tPC.Class]
  end
  if gDOMAIN_SKILLS > 0 then
    if tPC.Class=="Rogue" then
      for i = 1, #skill,1 do
        if isproficient(tPC, skill[i].Name) and not safetest(tPC.Class_Skill_Expertise[skill[i].Name], "Yes") then available[i] = "Yes" else available[i] = "No" end
      end
      table.insert(skill, {Name="Thieves Tools"})
      if isproficient(tPC, "Thieves Tools") and not safetest(tPC.Class_Tool_Expertise["Thieves Tools"], "Yes") then table.insert(available, "Yes") else table.insert(available, "No") end
    elseif safetest(tPC.Cleric_Domain, "Knowledge") then available = {"No", "No", "Yes", "No", "No", "Yes", "No", "No", "No", "No", "Yes", "No", "No", "No", "Yes", "No", "No", "No"}
    elseif safetest(tPC.Cleric_Domain, "Nature") then available = {"No", "Yes", "No", "No", "No", "No", "No", "No", "No", "No", "Yes", "No", "No", "No", "No", "No", "No", "Yes"}
    else end
  end
  for i= 1, #skill, 1 do
    if tPC.Class_Skill_Proficiency[skill[i].Name]=="Yes" then num = num + 1 end
  end
  if tPC.Class=="Rogue" and gDOMAIN_SKILLS > 0 then
    if safetest(tPC.Class_Tool_Expertise["Thieves Tools"], "Yes") then num = num + 1 end
    for i = 1, #skill, 1 do
      if safetest(tPC.Class_Skill_Expertise[skill[i].Name], "Yes") then num = num + 1 end
    end
  end
  if gDOMAIN_SKILLS > 0 then 
    if num<gCLASS_SKILLS+gDOMAIN_SKILLS-1 then title = "Choose "..tostring(gCLASS_SKILLS+gDOMAIN_SKILLS-num).." Expertise"
    else title = "Choose 1 Expertise" end
  elseif num<gCLASS_SKILLS-1 then title = "Choose "..tostring(gCLASS_SKILLS-num).." Skills"
  else title = "Choose 1 Skill" end
  for i = 1, #skill, 1 do
    if tPC.Class=="Rogue" and gDOMAIN_SKILLS > 0 and safetest(tPC.Class_Skill_Expertise[skill[i].Name], "Yes") then table.insert(buttons, {Text=skill[i].Name, Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
    elseif isproficient(tPC, skill[i].Name) and (tPC.Class~="Rogue" or gDOMAIN_SKILLS == 0) then table.insert(buttons, {Text=skill[i].Name, Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
    else
      if available[i]=="Yes" then table.insert(buttons, {Text=skill[i].Name, Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_class_skill", var=skill[i].Name}})
      else table.insert(buttons, {Text=skill[i].Name, Options={image=iICON["button_off"], font=fSUBHEADING, width=100, t=80}}) end
    end
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  createmenu(title, {buttons})
end

function click_class_skill(item)  -- Clicked a class skill
  local skill = {"Acrobatics", "Animal Handling", "Arcana", "Athletics", "Deception", "History", "Insight", "Intimidation", "Investigation", "Medicine", "Nature", "Perception", "Performance", "Persuasion", "Religion", "Sleight of Hand", "Stealth", "Survival"}
  local num = 0
  local n
  if gDOMAIN_SKILLS > 0 and tPC.Class == "Rogue" then
    if item == "Thieves Tools" then tPC.Class_Tool_Expertise["Thieves Tools"] = "Yes"
    else tPC.Class_Skill_Expertise[item] = "Yes" end
  else    
    tPC.Class_Skill_Proficiency[item] = "Yes"
  end
  if safetest(tPC.Cleric_Domain, "Knowledge") and gDOMAIN_SKILLS > 0 then tPC.Class_Skill_Expertise[item] = "Yes" end
  for n = 1, #skill, 1 do
    if safetest(tPC.Class_Skill_Proficiency[skill[n]], "Yes") then num = num + 1 end
  end
  if tPC.Class=="Rogue" and gDOMAIN_SKILLS > 0 then
    if safetest(tPC.Class_Tool_Expertise["Thieves Tools"], "Yes") then num = num + 1 end
    for n = 1, #skill, 1 do
      if safetest(tPC.Class_Skill_Expertise[skill[n]], "Yes") then num = num + 1 end
    end
  end
  if num < gCLASS_SKILLS+gDOMAIN_SKILLS then setup_class_skills()
  else 
    if (tPC.Class == "Rogue" or safetest(tPC.Cleric_Domain, "Knowledge")) and gDOMAIN_SKILLS == 0 then
      gDOMAIN_SKILLS = 2
      setup_class_skills()
    elseif safetest(tPC.Cleric_Domain, "Nature") and gDOMAIN_SKILLS == 0 then gDOMAIN_SKILLS = 1
    else setup_class_equipment() end
  end
end

function setup_class_equipment()  -- setup class choices for equipment
  local buttons = {}
  local i, n, x
  gSUBMENU = "create_equipment"
  tPC.Inventory = {}
  tPC.Equipped = {}
  tBUTTON = {}
  if next(tPC.Class_Equipment)~=nil then
    for i = 1, #tPC.Class_Equipment, 1 do
      buttons[i] = {}
      n = gettablesize(tPC.Class_Equipment[i])
      for x = 1, n, 1 do
        if x==1 then table.insert(buttons[i], {Text=tPC.Class_Equipment[i][x], Options={image=iICON["button_on"], font=fSUBHEADING, width=100}})
        else table.insert(buttons[i], {Text=tPC.Class_Equipment[i][x], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_class_equipment", var=tPC.Class_Equipment[i][x]}}) end
      end
      if n==3 then table.insert(buttons[i], {}) end
    end
  end
  table.insert(buttons, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_classmenu"}}, {Text="Accept", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="confirm_inventory"}}})
  createmenu("Choose Class Equipment", buttons)
end

function click_class_equipment(item)  -- choose this equipment for class
  local row = 0
  local i, j
  if next(tPC.Class_Equipment)~=nil then
    for i = 1, #tPC.Class_Equipment, 1 do
      for j = 1, #tPC.Class_Equipment[i], 1 do
        if tPC.Class_Equipment[i][j] == item then row = i end
      end
    end
  end
  if row ~= 0 then
    for i = 1, #tBUTTON, 1 do
      if tBUTTON[i].Text == item then
        tBUTTON[i].Image = iICON["button_on"]
        tBUTTON[i].Clickable = "No"
      else
        for j = 1, #tPC.Class_Equipment[row], 1 do
          if tBUTTON[i].Text == tPC.Class_Equipment[row][j] then
            tBUTTON[i].Image = iICON["button_off"]
            tBUTTON[i].Alt = iICON["button_on"]
            tBUTTON[i].Clickable = "Yes"
            tBUTTON[i].Script = "click_class_equipment"
            tBUTTON[i].Var = tPC.Class_Equipment[row][j]
          end
        end
      end
    end
  end
end

function confirm_inventory()  -- click accept inventory, setup ability buttons
  local i
  if next(tBUTTON)~=nil then
    for i = 1, #tBUTTON, 1 do
      if tBUTTON[i].Image == iICON["button_on"] and tBUTTON[i].Text~="Accept" then
        addinventoryitem(tPC, tBUTTON[i].Text)
      end
    end
  end
  setup_abilities()
end

function click_cleric()  -- click cleric, choose domain
  tPC.Class = "Cleric"
  tPC.HitDice = "1d8"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Medium Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Shield"] = "Yes"
  simpleweaponprof()
  tPC.Saving_Throw_Proficiency["Wisdom"] = "Yes"
  tPC.Saving_Throw_Proficiency["Charisma"] = "Yes"
  tPC.Class_Equipment = {{"Mace", "Warhammer"}, {"Scale Mail", "Leather Armor", "Chain Mail"}, {"Light Crossbow", "Handaxe"}, {"Priest Pack", "Explorer Pack"}, {"Shield"}, {"Holy Symbol"}}
  gCLASS_SKILLS = 2
  tSPELLS_TO_PICK = {"Cleric0", "Cleric0", "Cleric0"}
  tSPELL_SLOTS[1] = 2
  setup_clericdomain()
end

function setup_clericdomain()  -- setup cleric domain buttons
  local domain = {"Knowledge", "Life", "Light", "Nature", "Tempest", "Trickery", "War"}
  local buttons = {}
  local i
  gSUBMENU = "create_clericdomain"
  tBUTTON = {}
  for i = 1, #domain, 1 do
    table.insert(buttons, {Text=domain[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_clericdomain", var=domain[i]}})
  end
  table.insert(buttons, {})
  createmenu("Choose a Domain", {buttons})
end

function click_clericdomain(domain)  -- selected cleric domain, choose skills
  tPC.Cleric_Domain = domain
  if domain == "Knowledge" then gCLASS_LANGUAGE = 2
  elseif domain == "Life" then 
    tPC.Class_Armor_Proficiency["Heavy Armor"] = "Yes"
    tPC.Class_Feat["Disciple of Life"] = "Yes"
  elseif domain == "Light" then
    tPC.SpellKnown["Light"] = "Yes"
    tPC.Class_Feat["Warding Flare"] = "Yes"
  elseif domain == "Nature" then 
    table.insert(tSPELLS_TO_PICK, "Druid0")
  elseif domain == "Tempest" then
    tPC.Class_Armor_Proficiency["Heavy Armor"] = "Yes"
    martialweaponprof()
    tPC.Class_Feat["Wrath of the Storm"] = "Yes"
  elseif domain == "Trickery" then
    tPC.Class_Feat["Blessing of the Trickster"] = "Yes"
  elseif domain == "War" then
    tPC.Class_Armor_Proficiency["Heavy Armor"] = "Yes"
    martialweaponprof()
    tPC.Class_Feat["War Priest"] = "Yes"
  else end
  setup_class_skills()
end

function click_druid() -- click druid, choose skills
  tPC.Class = "Druid"
  tPC.HitDice = "1d8"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Medium Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Shield"] = "Yes"
  tPC.Class_Weapon_Proficiency["Club"] = "Yes"
  tPC.Class_Weapon_Proficiency["Dagger"] = "Yes"
  tPC.Class_Weapon_Proficiency["Dart"] = "Yes"
  tPC.Class_Weapon_Proficiency["Javelin"] = "Yes"
  tPC.Class_Weapon_Proficiency["Mace"] = "Yes"
  tPC.Class_Weapon_Proficiency["Quarterstaff"] = "Yes"
  tPC.Class_Weapon_Proficiency["Scimitar"] = "Yes"
  tPC.Class_Weapon_Proficiency["Sickle"] = "Yes"
  tPC.Class_Weapon_Proficiency["Sling"] = "Yes"
  tPC.Class_Weapon_Proficiency["Spear"] = "Yes"
  tPC.Class_Tool_Proficiency["Herbalism Kit"] = "Yes"
  tPC.Saving_Throw_Proficiency["Intelligence"] = "Yes"
  tPC.Saving_Throw_Proficiency["Wisdom"] = "Yes"
  tPC.Class_Equipment = {{"Wooden Shield", "Quarterstaff", "Spear"}, {"Scimitar", "Club", "Sling"}, {"Leather Armor"}, {"Explorer Pack"}, {"Druidic Focus"}}
  tPC.Class_Language["Druidic"] = "Yes"
  tSPELLS_TO_PICK = {"Druid0", "Druid0"}
  gCLASS_SKILLS = 2
  setup_class_skills()
end

function click_fighter()  -- click fighter, choose fighting style
  tPC.Class = "Fighter"
  tPC.HitDice = "1d10"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Medium Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Heavy_Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Shield"] = "Yes"
  simpleweaponprof()
  martialweaponprof()
  tPC.Saving_Throw_Proficiency["Strength"] = "Yes"
  tPC.Saving_Throw_Proficiency["Constitution"] = "Yes"
  tPC.Class_Equipment = {{"Chain Mail", "Leather, Longbow+Arrows"}, {"Longsword+Shield", "Battleaxe+Shield", "Two Scimitars"}, {"Light Crossbow+Bolts", "Two Handaxes"}, {"Dungeoneer Pack", "Explorer Pack"}}
  tPC.Class_Feat["Second Wind"] = "Yes"
  setup_fighting_styles()
end

function setup_fighting_styles()  -- setup fighting style buttons
  local style = {"Archery", "Defense", "Dueling", "Great Weapon Fighting", "Protection", "Two-Weapon Fighting"}
  local buttons = {}
  local i
  gSUBMENU = "create_fightingstyle"
  tBUTTON = {}
  for i = 1, #style, 1 do
    table.insert(buttons, {Text=style[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_fighting_style", var=style[i]}})
  end
  createmenu("Choose a Fighting Style", {buttons})
end

function click_fighting_style(item)  -- Select fighting style
  tPC.Class_Feat[item] = "Yes"
  setup_class_skills()
end

function click_monk()  -- chose monk, setup tool/instrument choice
  tPC.Class = "Monk"
  tPC.HitDice = "1d8"
  simpleweaponprof()
  tPC.Class_Weapon_Proficiency["Shortsword"] = "Yes"
  tPC.Saving_Throw_Proficiency["Strength"] = "Yes"
  tPC.Saving_Throw_Proficiency["Dexterity"] = "Yes"
  tPC.Class_Feat["Unarmored Defense"] = "Yes"
  tPC.Class_Feat["Martial Arts"] = "Yes"
  tPC.Class_Equipment = {{"Shortsword", "Quarterstaff", "Sickle"}, {"Dungeoneer Pack", "Explorer Pack"}, {"10 Darts"}}
  gCLASS_SKILLS = 2
  gCLASS_TOOLS = 1
  tBUTTON = {}
  createmenu("Choose a Proficiency Type", {{{Text="Artisan Tool", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_class_tools"}}, {Text="Mus. Instrument", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_musicbuttons"}}}})
end

function setup_class_tools()  -- artisan tools
  local title, buttons = "", {}
  local num = 0
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  gSUBMENU = "create_tools"
  tBUTTON = {}
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Artisan Tool") and tPC.Class_Tool_Proficiency[item[i].Name]=="Yes" then num = num + 1 end
  end
  if num<gCLASS_TOOLS-1 then title="Choose "..tostring(gCLASS_TOOLS-num).." Proficiencies"
  else title="Choose 1 Proficiency" end
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Artisan Tool") then
      if isproficient(tPC, item[i].Name) then table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
      else table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_tool", var=item[i].Name}}) end
    end
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  createmenu(title, {buttons})
end

function click_tool(name)   -- clicked a bard instrument, proficiency
  local num = 0
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  tPC.Class_Tool_Proficiency[name] = "Yes"
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Artisan Tool") and tPC.Class_Tool_Proficiency[item[i].Name]=="Yes" then num = num + 1 end
  end
  if num < gCLASS_TOOLS then setup_class_tools()
  else setup_class_skills() end
end

function click_paladin()  -- clicked paladin, choose skills
  tPC.Class = "Paladin"
  tPC.HitDice = "1d10"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Medium Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Heavy Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Shield"] = "Yes"
  simpleweaponprof()
  martialweaponprof()
  tPC.Saving_Throw_Proficiency["Wisdom"] = "Yes"
  tPC.Saving_Throw_Proficiency["Charisma"] = "Yes"
  tPC.Class_Feat["Divine Sense"] = "Yes"
  tPC.Class_Feat["Lay on Hands"] = "Yes"
  tPC.Class_Equipment = {{"Longsword+Shield", "Battleaxe+Shield", "Flail+Morningstar"}, {"Five Javelins", "Light Hammer", "Spear"}, {"Priest Pack", "Explorer Pack"}, {"Chain Mail"}, {"Holy Symbol"}}
  gCLASS_SKILLS = 2
  setup_class_skills()
end

function click_ranger()  -- clicked ranger, choose skills
  tPC.Class = "Ranger"
  tPC.HitDice = "1d10"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Medium Armor"] = "Yes"
  tPC.Class_Armor_Proficiency["Shield"] = "Yes"
  simpleweaponprof()
  martialweaponprof()
  tPC.Saving_Throw_Proficiency["Strength"] = "Yes"
  tPC.Saving_Throw_Proficiency["Dexterity"] = "Yes"
  tPC.Class_Equipment = {{"Scale Mail", "Leather Armor"}, {"Two Shortswords", "Quarterstaff+Spear", "Light Hammer+Mace"}, {"Dungeoneer Pack", "Explorer Pack"}, {"Longbow"}, {"20 Arrows"}}
  tPC.Class_Feat["Natural Explorer"] = "Yes"
  gCLASS_LANGUAGE = 1
  gCLASS_SKILLS = 3
  setup_class_skills()
end

function click_rogue()   -- clicked rogue
  tPC.Class = "Rogue"
  tPC.HitDice = "1d8"
  simpleweaponprof()
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  tPC.Class_Weapon_Proficiency["Hand Crossbow"] = "Yes"
  tPC.Class_Weapon_Proficiency["Longsword"] = "Yes"
  tPC.Class_Weapon_Proficiency["Rapier"] = "Yes"
  tPC.Class_Weapon_Proficiency["Shortsword"] = "Yes"
  tPC.Class_Tool_Proficiency["Thieves Tools"] = "Yes"
  tPC.Saving_Throw_Proficiency["Dexterity"] = "Yes"
  tPC.Saving_Throw_Proficiency["Intelligence"] = "Yes"
  tPC.Class_Equipment = {{"Rapier", "Shortsword"}, {"Shortbow+Arrows", "Shortsword"}, {"Burglar Pack", "Dungeoneer Pack", "Explorer Pack"}, {"Leather Armor"}, {"Two Daggers"}, {"Thieves Tools"}}
  tPC.Class_Feat["Sneak Attack"] = "Yes"
  tPC.Class_Language["Thieves Cant"] = "Yes"
  gCLASS_SKILLS = 4
  setup_class_skills()
end

function click_sorcerer()   -- clicked sorcerer
  tPC.Class = "Sorcerer"
  tPC.HitDice = "1d6"
  tPC.Class_Weapon_Proficiency["Dagger"] = "Yes"
  tPC.Class_Weapon_Proficiency["Dart"] = "Yes"
  tPC.Class_Weapon_Proficiency["Sling"] = "Yes"
  tPC.Class_Weapon_Proficiency["Quarterstaff"] = "Yes"
  tPC.Class_Weapon_Proficiency["Light Crossbow"] = "Yes"
  tPC.Saving_Throw_Proficiency["Constitution"] = "Yes"
  tPC.Saving_Throw_Proficiency["Charisma"] = "Yes"
  tPC.Class_Equipment = {{"Light Crossbow+Bolts", "Sling", "Quarterstaff"}, {"Component Pouch", "Arcane Focus"}, {"Dungeoneer Pack", "Explorer Pack"}, {"Two Daggers"}}
  gCLASS_SKILLS = 2
  tSPELLS_TO_PICK = {"Sorceror0", "Sorceror0", "Sorceror0", "Sorceror0", "Sorceror1", "Sorceror1"}
  tSPELL_SLOTS[1] = 2
  tBUTTON = {}
  createmenu("Choose a Sorcerous Origin", {{{Text="Draconic", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_sorcerer_draconic"}}, {Text="Wild Magic", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_sorcerer_wild"}}}})
end

function click_sorcerer_draconic()  -- clicked draconic sorcerous origin
  local dragon = {"Black", "Blue", "Brass", "Bronze", "Copper", "Gold", "Green", "Red", "Silver", "White"}
  local buttons = {}
  local i
  tPC.Class_Language["Draconic"] = "Yes"
  tPC.Sorcerous_Origin = "Draconic Bloodline"
  tPC.Class_Feat["Draconic Resilience"] = "Yes"
  tBUTTON = {}
  for i = 1, #dragon, 1 do
    table.insert(buttons, {Text=dragon[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_draconic_bloodline", var=dragon[i]}})
  end
  createmenu("Choose Draconic Bloodline", {buttons})
end

function click_draconic_bloodline(dragon)   -- chosen draconic bloodline
  tPC.Draconic_Bloodline = dragon
  setup_class_skills()
end

function click_sorcerer_wild()   -- clicked wild mage sorcerous origin
  tPC.Sorcerous_Origin = "Wild Magic"
  tPC.Class_Feat["Wild Magic Surge"] = "Yes"
  tPC.Class_Feat["Tides of Chaos"] = "Yes"
  setup_class_skills()
end

function click_warlock()  -- chose warlock, now choose patron
  tPC.Class = "Warlock"
  tPC.HitDice = "1d8"
  tPC.Class_Armor_Proficiency["Light Armor"] = "Yes"
  simpleweaponprof()
  tPC.Saving_Throw_Proficiency["Wisdom"] = "Yes"
  tPC.Saving_Throw_Proficiency["Charisma"] = "Yes"
  tPC.Class_Equipment = {{"Light Crossbow+Bolts", "Sling", "Quarterstaff"}, {"Component Pouch", "Arcane Focus"}, {"Scholar Pack", "Dungeoneer Pack"}, {"Leather Armor"}, {"Greatclub", "Spear", "Shortbow"}, {"Two Daggers"}}
  tSPELLS_TO_PICK = {"Warlock0", "Warlock0", "Warlock1", "Warlock1"}
  tSPELL_SLOTS[1] = 1
  gCLASS_SKILLS = 2
  tBUTTON = {}
  createmenu("Choose a Patron", {{{Text="The Archfey", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_warlock_patron", var="The Archfey"}}, {Text="The Fiend", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_warlock_patron", var="The Fiend"}}, {Text="Great Old One", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_warlock_patron", var="Great Old One"}}}})
end

function click_warlock_patron(patron) -- chose patron, now choose skills
  tPC.Warlock_Patron = patron
  if patron == "The Archfey" then tPC.Class_Feat["Fey Presence"] = "Yes"
  elseif patron == "The Fiend" then tPC.Class_Feat["Dark One Blessing"] = "Yes"
  elseif patron == "The Great Old One" then tPC.Class_Feat["Awakened Mind"] = "Yes"
  else end
  setup_class_skills()
end

function click_wizard()  -- chose wizard, now choose skills
  tPC.Class = "Wizard"
  tPC.HitDice = "1d6"
  tPC.Class_Weapon_Proficiency["Dagger"] = "Yes"
  tPC.Class_Weapon_Proficiency["Dart"] = "Yes"
  tPC.Class_Weapon_Proficiency["Sling"] = "Yes"
  tPC.Class_Weapon_Proficiency["Quarterstaff"] = "Yes"
  tPC.Class_Weapon_Proficiency["Light Crossbow"] = "Yes"
  tPC.Saving_Throw_Proficiency["Intelligence"] = "Yes"
  tPC.Saving_Throw_Proficiency["Wisdom"] = "Yes"
  tPC.Class_Equipment = {{"Quarterstaff", "Dagger"}, {"Component Pouch", "Arcane Focus"}, {"Scholar Pack", "Explorer Pack"}, {"Spellbook"}}
  tSPELLS_TO_PICK = {"Wizard0", "Wizard0", "Wizard0", "Wizard1", "Wizard1", "Wizard1", "Wizard1", "Wizard1", "Wizard1"}
  tSPELL_SLOTS[1] = 2
  tPC.Class_Feat["Arcane Recovery"] = "Yes"
  gCLASS_SKILLS = 2
  setup_class_skills()
end

function setup_abilities()  -- setup abilities
  gSUBMENU = "create_abilities"
  gPAGE = 0
  gABILITY = nil
  if tPC.Str~=nil then create_abilitybuttons()
  else click_reroll() end
end

function click_reroll()  -- reroll abilities (or roll for 1st time)
  local ability = {"Strength", "Dexterity", "Constitution", "Intelligence", "Wisdom", "Charisma"}
  local i
  for i = 1, 6, 1 do
    tPC[string.sub(ability[i],1,3)] = roll("4d6", 1)
  end
  playsound("dice", "Yes") 
  love.event.wait(0.5)
  create_abilitybuttons()
end

function create_abilitybuttons()  -- ability buttons
  local height, spacing = (gCANVAS_H-80)/10, (gCANVAS_H-80)/8
  local y = height+10
  local ability = {"Strength", "Dexterity", "Constitution", "Intelligence", "Wisdom", "Charisma"}
  local i
  tBUTTON = {}
  createbutton("Ability Scores", gCANVAS_W*0.5, y, {g=215, b=0})
  y = y + spacing
  for i = 1, 6, 1 do
    createbutton(ability[i], (gCANVAS_W*0.5)-58, y, {image=iICON["button_off"], font=fSUBHEADING, width=100, g=215, b=0})
    if gPAGE == #tBUTTON + 1 then createbutton(tPC[string.sub(ability[i],1,3)], (gCANVAS_W*0.5)+13, y, {image=iICON["small_on"], clickable="Yes", alt_image=iICON["small_on"], script="click_ability", var=string.sub(ability[i],1,3)})
    else createbutton(tPC[string.sub(ability[i],1,3)], (gCANVAS_W*0.5)+13, y, {image=iICON["small_off"], clickable="Yes", alt_image=iICON["small_on"], script="click_ability", var=string.sub(ability[i],1,3)}) end
    createbutton("+"..tPC[string.sub(ability[i],1,3).."_RacBonus"], (gCANVAS_W*0.5)+51, y, {image=iICON["small_off"], g=215, b=0})
    createbutton(getabilityscore(tPC, ability[i]), (gCANVAS_W*0.5)+89, y, {image=iICON["small_off"], g=215, b=0})
    y = y + spacing
  end
  createbutton("Back", (gCANVAS_W*0.5)-86, y, {image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="setup_classmenu"})
  createbutton("Re-Roll", gCANVAS_W*0.5, y, {image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="click_reroll"})
  createbutton("Accept", (gCANVAS_W*0.5)+86, y, {image=iICON["button_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["button_on"], script="setup_backgroundmenu"})
end

function click_ability(ability)  -- select or swap ability score
  local mx, my = love.mouse.getX(), love.mouse.getY()
  local i, clicked
  if gORIENTATION == "portrait" then mx,my = (gSCREEN_W-my)/gRATIOX, mx/gRATIOY
  else mx,my = mx/gRATIOX, my/gRATIOY end
  clicked = getbuttonclicked(mx, my)
  if clicked ~= 0 then
    if gPAGE == 0 then 
      gPAGE = clicked
      gABILITY = ability
    else
      i = tPC[ability]
      tPC[ability] = tPC[gABILITY]
      tPC[gABILITY] = i
      gPAGE = 0
      gABILITY = nil
    end
  end
  create_abilitybuttons()
end

function roll(text, drop_lowest, drop_highest)  -- roll ndx +- z
  local rolls = {}
  local r = 0
  local i
  local num, dice, modify = interpretdice(text)
  drop_lowest, drop_highest = drop_lowest or 0, drop_highest or 0
  if num>=1 then
    for i = 1, num, 1 do
      if dice>1 then table.insert(rolls, math.random(dice))
      else table.insert(rolls, 1) end
    end
    table.sort(rolls)
    if drop_lowest >= 1 and #rolls>drop_lowest then
      for i = 1, drop_lowest, 1 do
        table.remove(rolls, 1)
      end
    end
    if drop_highest >= 1 and #rolls>drop_highest then
      for i = 1, drop_highest, 1 do
        table.remove(rolls)
      end
    end
    for i = 1, #rolls, 1 do
      r = r + rolls[i]
    end
  end
  r = r + modify
  return r
end

function interpretdice(text)  -- interpret ndx +- z
  local num, dice, modify = 0, 1, 0
  local k = string.len(text)
  local i, j
  if k>=3 and string.find(text, "d") and string.sub(text, string.len(text)-1, string.len(text))~="d" then
    num = tonumber(string.sub(string.match(text, "%d*d"), 1, string.len(string.match(text, "%d*d"))-1))
    if string.find(text, "+") and not string.find(text, "-") then
      dice = tonumber(string.sub(string.match(text, "d%d*%+"), 2, string.len(string.match(text, "d%d*%+"))-1))
      i, j = string.find(text, "+")
      if string.len(text)>j then modify = tonumber(string.match(string.sub(text, j, string.len(text)), "%+%d*")) else modify = 0 end
    elseif string.find(text, "-") and not string.find(text, "+") then
      dice = tonumber(string.sub(string.match(text, "d%d*%-"), 2, string.len(string.match(text, "d%d*%-"))-1))
      i, j = string.find(text, "+")
      if string.len(text)>j then modify = -tonumber(string.match(string.sub(text, j, string.len(text)), "%-%d*")) else modify = 0 end
    else
      dice = tonumber(string.sub(string.match(text, "d%d*"), 2, string.len(string.match(text, "d%d*"))))
      modify = 0 
    end
  end
  return num, dice, modify
end

function setup_backgroundmenu()  -- choose a background
  local background = readsinglelist("data/backgrounds.txt")
  local buttons = {}
  local i
  gSUBMENU = "create_background"
  tPC.Background = nil
  tPC.Background_Skill_Proficiency = {}
  tPC.Background_Tool_Proficiency = {}
  tPC.Background_Feat = {}
  tPC.Gold = 0
  gBACKGROUND_LANGUAGE = 0
  if tRACE_PICKED_LANGUAGE~=nil then
    while gettablesize(tRACE_PICKED_LANGUAGE) > 0 do
      tPC.Race_Language[tRACE_PICKED_LANGUAGE[1]] = nil
      table.remove(tRACE_PICKED_LANGUAGE, 1)
    end
  end
  tBUTTON = {}
  for i = 1, #background, 1 do
    table.insert(buttons, {Text=background[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_"..trim_spaces(string.lower(background[i]))}})
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  createmenu("Choose a Background", {buttons, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="create_abilitybuttons"}}}})
end

function trim_spaces(s)  -- trim spaces in text
  return s:match("^%s*(.-)%s*$")
end

function click_acolyte()  -- acolyte background
  tPC.Background = "Acolyte"
  tPC.Background_Skill_Proficiency["Insight"] = "Yes"
  tPC.Background_Skill_Proficiency["Religion"] = "Yes"
  gBACKGROUND_LANGUAGE = 2
  addinventoryitem(tPC, "Holy Symbol")
  addinventoryitem(tPC, "Prayer Book")
  addinventoryitem(tPC, "Incense Stick", 5)
  addinventoryitem(tPC, "Vestments")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Gold = 15
  setup_languagemenu()
end

function click_charlatan()  -- charlatan background
  tPC.Background = "Charlatan"
  tPC.Background_Skill_Proficiency["Deception"] = "Yes"
  tPC.Background_Skill_Proficiency["Sleight of Hand"] = "Yes"
  tPC.Background_Tool_Proficiency["Disguise Kit"] = "Yes"
  tPC.Background_Tool_Proficiency["Forgery Kit"] = "Yes"
  addinventoryitem(tPC, "Fine Clothes")
  addinventoryitem(tPC, "Disguise Kit")
  addinventoryitem(tPC, "Weighted Dice")
  addinventoryitem(tPC, "Fake Signet Ring")
  addinventoryitem(tPC, "Marked Cards")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Gold = 15
  setup_languagemenu()
end

function click_criminal()  -- criminal backgrouond
  tPC.Background_Skill_Proficiency["Deception"] = "Yes"
  tPC.Background_Skill_Proficiency["Stealth"] = "Yes"
  tPC.Background_Tool_Proficiency["Dice Set"] = "Yes"
  tPC.Background_Tool_Proficiency["Thieves Tools"] = "Yes"
  addinventoryitem(tPC, "Crowbar")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Hood")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Gold = 15
  setup_languagemenu()
end

function click_entertainer()  -- entertainer background
  tPC.Background_Skill_Proficiency["Acrobatics"] = "Yes"
  tPC.Background_Skill_Proficiency["Performance"] = "Yes"
  tPC.Background_Tool_Proficiency["Disguise Kit"] = "Yes"
  addinventoryitem(tPC, "Fan Letter")
  addinventoryitem(tPC, "Costume")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Gold = 15
  tPC.Background_Feat["By Popular Demand"] = "Yes"
  setup_background_instrument()
end

function background_tool(item)  -- choose instrument for entertainer/tool for folk hero
  tPC.Background_Tool_Proficiency[item] = "Yes"
  addinventoryitem(tPC, item)
  setup_languagemenu()
end

function click_folkhero()  -- Folk Hero background
  tPC.Background_Skill_Proficiency["Animal Handling"] = "Yes"
  tPC.Background_Skill_Proficiency["Survival"] = "Yes"
  addinventoryitem(tPC, "Shovel")
  addinventoryitem(tPC, "Iron Pot")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Gold = 10
  tPC.Background_Feat["Rustic Hospitality"] = "Yes"
  setup_background_tool()
end

function setup_background_tool()  -- background artisan tool buttons
  local buttons = {}
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  tBUTTON = {}
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Artisan Tool") then
      if isproficient(tPC, item[i].Name) then table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}}) else table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="background_tool", var=item[i].Name}}) end
    end
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  createmenu("Choose an Artisan Tool", {buttons})
end

function click_guildartisan()  -- Guild Artisan background
  tPC.Background_Skill_Proficiency["Insight"] = "Yes"
  tPC.Background_Skill_Proficiency["Persuasion"] = "Yes"
  gBACKGROUND_LANGUAGE = 1
  addinventoryitem(tPC, "Letter of Introduction")
  addinventoryitem(tPC, "Traveler Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Background_Feat["Guild Membership"] = "Yes"
  tPC.Gold = 15
  setup_background_tool()
end

function click_hermit()   -- Hermit background
  tPC.Background_Skill_Proficiency["Medicine"] = "Yes"
  tPC.Background_Skill_Proficiency["Religion"] = "Yes"
  tPC.Background_Tool_Proficiency["Herbalism Kit"] = "Yes"
  gBACKGROUND_LANGUAGE = 1
  addinventoryitem(tPC, "Scroll Case")
  addinventoryitem(tPC, "Winter Blanket")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Herbalism Kit")
  tPC.Background_Feat["Discovery"] = "Yes"
  tPC.Gold = 5
  setup_languagemenu()
end

function click_noble()  -- Noble background
  tPC.Background_Skill_Proficiency["History"] = "Yes"
  tPC.Background_Skill_Proficiency["Persuasion"] = "Yes"
  tPC.Background_Tool_Proficiency["Cards"] = "Yes"
  gBACKGROUND_LANGUAGE = 1
  addinventoryitem(tPC, "Fine Clothes")
  addinventoryitem(tPC, "Signet Ring")
  addinventoryitem(tPC, "Scroll of Pedigree")
  addinventoryitem(tPC, "Purse")
  tPC.Background_Feat["Position of Privilege"] = "Yes"
  tPC.Gold = 25
  setup_languagemenu()
end

function click_outlander()  -- Outlander background
  tPC.Background_Skill_Proficiency["Athletics"] = "Yes"
  tPC.Background_Skill_Proficiency["Survival"] = "Yes"
  gBACKGROUND_LANGUAGE = 1
  addinventoryitem(tPC, "Quarterstaff")
  addinventoryitem(tPC, "Hunting Trap")
  addinventoryitem(tPC, "Stag Antlers")
  addinventoryitem(tPC, "Traveler Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Background_Feat["Wanderer"] = "Yes"
  tPC.Gold = 10
  setup_background_instrument()
end

function click_sage()  -- Sage background
  tPC.Background_Skill_Proficiency["Arcana"] = "Yes"
  tPC.Background_Skill_Proficiency["History"] = "Yes"
  gBACKGROUND_LANGUAGE = 2
  addinventoryitem(tPC, "Bottle of Ink")
  addinventoryitem(tPC, "Quill")
  addinventoryitem(tPC, "Small Knife")
  addinventoryitem(tPC, "Parchment")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Background_Feat["Researcher"] = "Yes"
  tPC.Gold = 10
  setup_languagemenu()
end

function click_sailor()  -- Sailor background
  tPC.Background_Skill_Proficiency["Athletics"] = "Yes"
  tPC.Background_Skill_Proficiency["Perception"] = "Yes"
  tPC.Background_Tool_Proficiency["Navigator Tools"] = "Yes"
  addinventoryitem(tPC, "Belaying Pin")
  addinventoryitem(tPC, "Silk Rope")
  addinventoryitem(tPC, "Trinket")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Background_Feat["Ship Passage"] = "Yes"
  tPC.Gold = 10
  setup_languagemenu()
end

function click_soldier()  -- Soldier background
  tPC.Background_Skill_Proficiency["Athletics"] = "Yes"
  tPC.Background_Skill_Proficiency["Intimidation"] = "Yes"
  tPC.Background_Tool_Proficiency["Cards"] = "Yes"
  addinventoryitem(tPC, "Rank Insignia")
  addinventoryitem(tPC, "Dagger")
  addinventoryitem(tPC, "Cards")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Background_Feat["Military Rank"] = "Yes"
  tPC.Gold = 10
  setup_languagemenu()
end

function click_urchin()  -- Urchin background
  tPC.Background_Skill_Proficiency["Sleight of Hand"] = "Yes"
  tPC.Background_Skill_Proficiency["Stealth"] = "Yes"
  tPC.Background_Tool_Proficiency["Disguise Kit"] = "Yes"
  tPC.Background_Tool_Proficiency["Thieves Tools"] = "Yes"
  addinventoryitem(tPC, "Small Knife")
  addinventoryitem(tPC, "Map of City")
  addinventoryitem(tPC, "Pet Mouse")
  addinventoryitem(tPC, "Token")
  addinventoryitem(tPC, "Common Clothes")
  addinventoryitem(tPC, "Belt Pouch")
  tPC.Background_Feat["City Secrets"] = "Yes"
  tPC.Gold = 10
  setup_languagemenu()
end

function setup_background_instrument()  -- Background instrument buttons
  local buttons = {}
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  tBUTTON = {}
  for i = 1, #item, 1 do
    if safetest(item[i].Kind, "Instrument") then
      if isproficient(tPC, item[i].Name) then table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
      else table.insert(buttons, {Text=item[i].Name, Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="background_tool", var=item[i].Name}}) end
    end
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  createmenu("Choose a Musical Instrument", {buttons})
end

function addinventoryitem(creature, item, n)
  local i, list
  n = n or 1
  if creature.Inventory==nil then creature.Inventory = {} end
  if item == "Two Handaxes" then
    table.insert(creature.Inventory, "Handaxe")
    table.insert(creature.Inventory, "Handaxe")
  elseif item == "Four Javelins" then
    for i = 1, 4, 1 do
      table.insert(creature.Inventory, "Javelin")
    end
  elseif item == "Longbow+Arrows" then
    table.insert(creature.Inventory, "Longbow")
    table.insert(creature.Inventory, "Quiver")
    for i = 1, 20, 1 do
      table.insert(creature.Inventory, "Arrow")
    end
  elseif item == "Longsword+Shield" then
    table.insert(creature.Inventory, "Longsword")
    table.insert(creature.Inventory, "Shield")
  elseif item == "Battleaxe+Shield" then
    table.insert(creature.Inventory, "Battleaxe")
    table.insert(creature.Inventory, "Shield")
  elseif item == "Two Scimitars" then
    table.insert(creature.Inventory, "Scimitar")
    table.insert(creature.Inventory, "Scimitar")
  elseif item == "Light Crossbow+Bolts" then
    table.insert(creature.Inventory, "Light Crossbow")
    table.insert(creature.Inventory, "Quiver")    
    for i = 1, 20, 1 do
      table.insert(creature.Inventory, "Bolt")
    end
  elseif item == "10 Darts" then
    for i = 1, 10, 1 do
      table.insert(creature.Inventory, "Dart")
    end
  elseif item == "Flail+Morningstar" then 
    table.insert(creature.Inventory, "Flail")
    table.insert(creature.Inventory, "Morningstar")
  elseif item == "Two Shortswords" then
    table.insert(creature.Inventory, "Shortsword")
    table.insert(creature.Inventory, "Shortsword")  
  elseif item == "Quarterstaff+Spear" then
    table.insert(creature.Inventory, "Quarterstaff")
    table.insert(creature.Inventory, "Spear")
  elseif item == "Light Hammer+Mace" then
    table.insert(creature.Inventory, "Light Hammer")
    table.insert(creature.Inventory, "Mace")
  elseif item == "20 Arrows" then
    for i = 1, 20, 1 do
      table.insert(creature.Inventory, "Arrow")
    end
  elseif item == "Shortbow+Arrows" then
    table.insert(creature.Inventory, "Shortbow")
    table.insert(creature.Inventory, "Quiver")
    for i = 1, 20, 1 do
      table.insert(creature.Inventory, "Arrow")
    end
  elseif item == "Two Daggers" then
    table.insert(creature.Inventory, "Dagger")
    table.insert(creature.Inventory, "Dagger")
  elseif item == "Explorer Pack" then
    list = readsinglelist("data/pack_explorer.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  elseif item == "Diplomat Pack" then
    list = readsinglelist("data/pack_diplomat.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  elseif item == "Entertainer Pack" then
    list = readsinglelist("data/pack_entertainer.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  elseif item == "Priest Pack" then
    list = readsinglelist("data/pack_priest.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  elseif item == "Dungeoneer Pack" then
    list = readsinglelist("data/pack_dungeoneer.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  elseif item == "Burglar Pack" then
    list = readsinglelist("data/pack_burglar.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  elseif item == "Scholar Pack" then
    list = readsinglelist("data/pack_scholar.txt")
    if next(list)~=nil then
      for i = 1, #list, 1 do
        table.insert(creature.Inventory, list[i])
      end
    end
  else
    for i = 1, n, 1 do
      table.insert(creature.Inventory, item)
    end
  end

end

function setup_languagemenu()  -- setup language menu
  gSUBMENU = "create_languages"
  tPICKED_RACE_LANGUAGE = {}
  tPICKED_BACKGROUND_LANGUAGE = {}
  tPICKED_CLASS_LANGUAGE = {}
  tPC.Background_Language = {}
  if gRACE_LANGUAGE+gCLASS_LANGUAGE+gBACKGROUND_LANGUAGE > 0 then create_languagebuttons()
  else setup_spells() end
end

function create_languagebuttons()  -- create language buttons
  local language = readsinglelist("data/languages.txt")
  local buttons = {}
  local i
  tBUTTON = {}
  for i = 1, #language, 1 do
    if not speakslanguage(tPC, language[i]) then
      table.insert(buttons, {Text=language[i], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="addlanguage", var=language[i]}})
    else
      table.insert(buttons, {Text=language[i], Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
    end
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  createmenu("Choose "..gRACE_LANGUAGE+gBACKGROUND_LANGUAGE+gCLASS_LANGUAGE.." Languages", {buttons, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_backgroundmenu"}}}})
end

function speakslanguage(creature, lang)  -- does the creature speak this language?
  local speak = false
  if next(creature)~=nil then
    if creature.Race_Language~=nil and safetest(creature.Race_Language[lang], "Yes") then speak = true end
    if creature.Background_Language~=nil and safetest(creature.Background_Language[lang], "Yes") then speak = true end
    if creature.Class_Language~=nil and safetest(creature.Class_Language[lang], "Yes") then speak = true end
  end
  return speak
end

function addlanguage(lang)  -- player chose language
  if gRACE_LANGUAGE > 0 then
    tPC.Race_Language[lang] = "Yes"
    table.insert(tPICKED_RACE_LANGUAGE, lang)
    gRACE_LANGUAGE = gRACE_LANGUAGE - 1
  elseif gBACKGROUND_LANGUAGE > 0 then
    tPC.Background_Language[lang] = "Yes"
    table.insert(tPICKED_BACKGROUND_LANGUAGE, lang)
    gBACKGROUND_LANGUAGE = gBACKGROUND_LANGUAGE - 1
  elseif gCLASS_LANGUAGE > 0 then
    tPC.Class_Language[lang] = "Yes"
    table.insert(tPICKED_CLASS_LANGUAGE, lang)
    gCLASS_LANGUAGE = gCLASS_LANGUAGE - 1
  else 
    -- How did they get to choose a language?
  end
  if gRACE_LANGUAGE+gBACKGROUND_LANGUAGE+gCLASS_LANGUAGE > 0 then create_languagebuttons()
  else setup_spells() end
end

function setup_spells()  -- setup the spells menu, or skip to customise character
  gSUBMENU = "create_spells"
  tPICKED_SPELLS = {}
  gPAGE = 1
  if needpickspells() then create_spellbuttons()
  else setup_customise() end
end

function needpickspells()  -- does the player need to select more spells?
  local pick = false
  if gRACE_WIZARDCANTRIP~=nil and gRACE_WIZARDCANTRIP>0 then pick = true
  elseif tSPELLS_TO_PICK~=nil and #tSPELLS_TO_PICK>0 then pick = true
  else end
  return pick
end

function create_spellbuttons()  -- setup the buttons to select lowest level spells available
  local book = {"Bard", "Cleric", "Druid", "Paladin", "Ranger", "Sorceror", "Warlock", "Wizard"}
  local spells = readmultitable("data/spells.txt")
  local height, spacing = (gCANVAS_H-80)/10, (gCANVAS_H-80)/8
  local y = height+10
  local selection, buttons, pages = {}, {}, {}
  local n, text
  local lowest_book, lowest_level, num = get_lowestspell()
  tBUTTON = {}
  if lowest_book <= 8 and lowest_level <= 9 and num > 0 then
    text = "Choose "..num.." "..book[lowest_book]
    if lowest_level==0 then text = text.." Cantrips"
    else text = text.." Level "..lowest_level.." Spells" end
    if num == 1 then text = string.sub(text, 1, string.len(text)-1) end
  end
  for n = 1, #spells, 1 do
    if spells[n].Level==lowest_level and safetest(spells[n][book[lowest_book]], "Yes") then table.insert(selection, spells[n].Name) end
  end
  while ((gPAGE-1)*10)+1>#selection do
    gPAGE = gPAGE - 1
  end
  if #selection > 0 then
    for n = ((gPAGE-1)*10)+1, math.min(gPAGE*10, #selection), 1 do
      if safetest(tPC.SpellKnown[selection[n]], "Yes") then 
        table.insert(buttons, {Text=selection[n], Options={image=iICON["button_on"], font=fSUBHEADING, width=100, t=80}})
      else
        table.insert(buttons, {Text=selection[n], Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="pick_spell", var=selection[n]}})
      end
    end
  end
  if #buttons%2~=0 then table.insert(buttons, {}) end
  if gPAGE>1 then 
    table.insert(pages, {Text="<<", Options={image=iICON["small_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["small_on"], script="spell_page", var="left"}})
  else table.insert(pages, {Text="<<", Options={image=iICON["small_off"], font=fSUBHEADING, t=80}}) end
  if #selection>gPAGE*10 then 
    table.insert(pages, {Text=">>", Options={image=iICON["small_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["small_on"], script="spell_page", var="right"}})
  else table.insert(pages, {Text=">>", Options={image=iICON["small_off"], font=fSUBHEADING, t=80}}) end
  createmenu(text, {buttons, pages, {{Text="Back", Options={image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="click_newchar"}}}})
end

function get_lowestspell()  -- what's the lowest spell they can pick?
  local book = {"Bard", "Cleric", "Druid", "Paladin", "Ranger", "Sorceror", "Warlock", "Wizard"}
  local lowest_book, lowest_level, num = math.huge, math.huge, 0
  local i, j, n
  if gRACE_WIZARDCANTRIP > 0 then lowest_book, lowest_level = 8, 0 end
  if gettablesize(tSPELLS_TO_PICK) > 0 then
    for i = 1, #tSPELLS_TO_PICK, 1 do
      n = tonumber(string.sub(tSPELLS_TO_PICK[i], string.len(tSPELLS_TO_PICK[i]), string.len(tSPELLS_TO_PICK[i])))
      for j = #book, 1, -1 do
        if string.sub(tSPELLS_TO_PICK[i], 1, string.len(tSPELLS_TO_PICK[i])-1) == book[j] then 
          if j < lowest_book or (j==lowest_book and n < lowest_level) then lowest_book, lowest_level = j, n end
        end
      end
    end
    for i = 1, #tSPELLS_TO_PICK, 1 do
      n = tonumber(string.sub(tSPELLS_TO_PICK[i], string.len(tSPELLS_TO_PICK[i]), string.len(tSPELLS_TO_PICK[i])))
      if lowest_level == n and string.sub(tSPELLS_TO_PICK[i], 1, string.len(tSPELLS_TO_PICK[i])-1) == book[lowest_book] then num = num + 1 end
    end
  end
  if lowest_book == 8 and lowest_level == 0 and gRACE_WIZARDCANTRIP > 0 then num = num + gRACE_WIZARDCANTRIP end
  return lowest_book, lowest_level, num
end

function pick_spell(spell)  -- choose the spell, add it
  local book = {"Bard", "Cleric", "Druid", "Paladin", "Ranger", "Sorceror", "Warlock", "Wizard"}
  local lowest_book, lowest_level, num = get_lowestspell()
  local text = book[lowest_book]..lowest_level
  local i, found = 0, false
  tPC.SpellKnown[spell] = "Yes"
  while found == false and i<=#tSPELLS_TO_PICK do
    i = i + 1
    if tSPELLS_TO_PICK[i]==text then found = true end
  end
  if found == true then 
    table.insert(tPICKED_SPELLS, tSPELLS_TO_PICK[i])
    table.remove(tSPELLS_TO_PICK, i)
  elseif gRACE_WIZARDCANTRIP > 0 and lowest_book==8 and lowest_level==0 then
    table.insert(tPICKED_SPELLS, "RACE_WIZARDCANTRIP")
    gRACE_WIZARDCANTRIP = gRACE_WIZARDCANTRIP - 1
  else end
  setup_spells()
end

function spell_page(arrow)  -- next/previous page of spells to pick
  if arrow == "left" then gPAGE = gPAGE - 1
  elseif arrow == "right" then gPAGE = gPAGE + 1
  else end
  create_spellbuttons()
end

function setup_customise()  -- Customise character
  local eye = readsinglelist("data/eyecolour.txt")
  local hair = readsinglelist("data/haircolour.txt")
  local skin = readsinglelist("data/skincolour.txt")
  gSUBMENU = "create_customise"
  if tPC.EyeColour==nil then tPC.EyeColour = eye[math.random(#eye)] end
  if tPC.HairColour==nil then tPC.HairColour = hair[math.random(#hair)] end
  if tPC.SkinColour==nil then tPC.SkinColour = skin[math.random(#skin)] end
  if tPC.Deity==nil then tPC.Deity = "None" end
  customise_buttons()
end

function customise_buttons()  -- Customise height, weight etc.
  local stat = {"Height: "..inchestofeet(tPC.Height), "Weight: "..poundstostones(tPC.Weight), "Eyes: "..tPC.EyeColour, "Hair: "..tPC.HairColour, "Skin: "..tPC.SkinColour, "Deity: "..tPC.Deity, "Age: "..tPC.Age}
  local left_var = {"height_down", "weight_down", "eyes_prev", "hair_prev", "skin_prev", "deity_prev", "age_down"}
  local right_var = {"height_up", "weight_up", "eyes_next", "hair_next", "skin_next", "deity_next", "age_up"}
  local height, spacing = (gCANVAS_H-80)/10, (gCANVAS_H-80)/8
  local y = height+10
  local i
  tBUTTON = {}
  createbutton("Customise Character", gCANVAS_W*0.5, y, {g=215, b=0})
  y = y + spacing
  for i = 1, #stat, 1 do
    createbutton(stat[i], gCANVAS_W*0.5, y, {font=fSUBHEADING, g=215, b=0})
    createbutton("<<", (gCANVAS_W*0.5)-92, y, {image=iICON["small_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["small_on"], script="customise_stat", var=left_var[i]})
    createbutton(">>", (gCANVAS_W*0.5)+92, y, {image=iICON["small_off"], font=fSUBHEADING, clickable="Yes", alt_image=iICON["small_on"], script="customise_stat", var=right_var[i]})
    y = y + spacing
  end
  if next(tPICKED_SPELLS)~=nil then createbutton("Back", (gCANVAS_W*0.5)-60, y, {image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="undo_spells"})
  else createbutton("Back", (gCANVAS_W*0.5)-60, y, {image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="undo_languages"}) end
  createbutton("Accept", (gCANVAS_W*0.5)+60, y, {image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="setup_name"})
end

function customise_stat(action)  -- change something
  local eye = readsinglelist("data/eyecolour.txt")
  local hair = readsinglelist("data/haircolour.txt")
  local skin = readsinglelist("data/skincolour.txt")
  local deities = readmultitable("data/deities.txt")
  local deity = {}
  local i
  if next(deities)~=nil then
    for i = 1, #deities, 1 do
      deity[i] = deities[i].Name
    end
  end
  if action=="height_up" then tPC.Height = math.min(tPC.HeightMax, tPC.Height+1)
  elseif action=="height_down" then tPC.Height = math.max(tPC.HeightMin, tPC.Height-1)
  elseif action=="weight_up" then tPC.Weight = math.min(tPC.WeightMax, tPC.Weight+1)
  elseif action=="weight_down" then tPC.Weight = math.max(tPC.WeightMin, tPC.Weight-1)
  elseif action=="eyes_next" then tPC.EyeColour = getnextoption(tPC.EyeColour, eye)
  elseif action=="eyes_prev" then tPC.EyeColour = getpreviousoption(tPC.EyeColour, eye)
  elseif action=="hair_next" then tPC.HairColour = getnextoption(tPC.HairColour, hair)
  elseif action=="hair_prev" then tPC.HairColour = getpreviousoption(tPC.HairColour, hair)
  elseif action=="skin_next" then tPC.SkinColour = getnextoption(tPC.SkinColour, skin)
  elseif action=="skin_prev" then tPC.SkinColour = getpreviousoption(tPC.SkinColour, skin)
  elseif action=="deity_next" then tPC.Deity = getnextoption(tPC.Deity, deity)
  elseif action=="deity_prev" then tPC.Deity = getpreviousoption(tPC.Deity, deity)
  elseif action=="age_up" then tPC.Age = math.min(tPC.AgeMax, tPC.Age+1)
  elseif action=="age_down" then tPC.Age = math.max(tPC.AgeMin, tPC.Age-1)
  else end
  customise_buttons()
end

function undo_spells()  -- undo spell choices and go back
  tSPELLS_TO_PICK = {}
  gRACE_WIZARDCANTRIP = 0
  local i
  if next(tPICKED_SPELLS)~=nil then
    for i = #tPICKED_SPELLS, 1, -1 do
      if tPICKED_SPELLS[i]=="RACE_WIZARDCANTRIP" then gRACE_WIZARDCANTRIP = gRACE_WIZARDCANTRIP + 1
      else table.insert(tSPELLS_TO_PICK, tPICKED_SPELLS[i]) end
      table.remove(tPICKED_SPELLS, i)
    end
  end
  setup_spells()
end

function undo_languages()  -- undo language choices and go back
  gRACE_LANGUAGE, gCLASS_LANGUAGE, gBACKGROUND_LANGUAGE = 0, 0, 0
  local i
  if next(tPICKED_RACE_LANGUAGE)~=nil then
    for i = #tPICKED_RACE_LANGUAGE, 1, -1 do
      tPC.Race_Language[tPICKED_RACE_LANGUAGE[i]]=nil
      table.remove(tPICKED_RACE_LANGUAGE, i)
      gRACE_LANGUAGE = gRACE_LANGUAGE + 1
    end
  end
  if next(tPICKED_BACKGROUND_LANGUAGE)~=nil then
    for i = #tPICKED_BACKGROUND_LANGUAGE, 1, -1 do
      tPC.Background_Language[tPICKED_BACKGROUND_LANGUAGE[i]]=nil
      table.remove(tPICKED_BACKGROUND_LANGUAGE, i)
      gBACKGROUND_LANGUAGE = gBACKGROUND_LANGUAGE + 1
    end
  end
  if next(tPICKED_CLASS_LANGUAGE)~=nil then
    for i = #tPICKED_CLASS_LANGUAGE, 1, -1 do
      tPC.Race_Language[tPICKED_CLASS_LANGUAGE[i]]=nil
      table.remove(tPICKED_CLASS_LANGUAGE, i)
      gCLASS_LANGUAGE = gCLASS_LANGUAGE + 1
    end
  end
  setup_languagemenu()
end

function setup_name()  -- setup name button
  gSUBMENU = "create_name"
  settyping(tPC.Class)
  create_namebuttons()
end

function create_namebuttons()  -- create name button
  local height, spacing = (gCANVAS_H-80)/10, (gCANVAS_H-80)/8
  local y = height+10
  tBUTTON = {}
  createbutton("Name your character...", gCANVAS_W*0.5, y, {g=215, b=0})
  if gTYPING ~= true then createbutton(tPC.Name, gCANVAS_W*0.5, gCANVAS_H*0.5, {g=215, b=0, clickable="Yes", script="settyping", var=tPC.Name}) end
  createbutton("Accept", gCANVAS_W*0.5, (gCANVAS_H*0.5)+(3*y), {image=iICON["button_off"], font=fSUBHEADING, width=100, clickable="Yes", alt_image=iICON["button_on"], script="accept_name"})
end

function savecharacter(location)  -- Save character
  local table = {"Race_Feat", "Race_Armor_Proficiency", "Race_Weapon_Proficiency", "Race_Tool_Proficiency", "Race_Skill_Proficiency", "Race_Language", "SpellKnown", "Class_Weapon_Proficiency", "Class_Armor_Proficiency", "Class_Tool_Proficiency", "Class_Tool_Expertise", "Class_Skill_Proficiency", "Class_Skill_Expertise", "Saving_Throw_Proficiency", "Class_Feat", "Background_Skill_Proficiency", "Background_Tool_Proficiency", "Background_Feat", "Background_Language"}
  local list = {"Inventory", "Equipped"}
  local i
  if tPC.Name~=nil then
    location = location or "Autosave/Character/"..tPC.Name
    if love.filesystem.getInfo(location, "directory") then erasefolder(location) end
    love.filesystem.createDirectory(location)
    writesingletable(location.."/Char.txt", tPC)
    for i = 1, #table, 1 do
      writesingletable(location.."/"..table[i]..".txt", tPC[table[i]])
    end
    for i = 1, #list, 1 do
      writesinglelist(location.."/"..list[i]..".txt", tPC[list[i]])
    end
  end
end

function loadcharacter(location)  -- Load character by directory reference
  local table = {"Race_Feat", "Race_Armor_Proficiency", "Race_Weapon_Proficiency", "Race_Tool_Proficiency", "Race_Skill_Proficiency", "Race_Language", "SpellKnown", "Class_Weapon_Proficiency", "Class_Armor_Proficiency", "Class_Tool_Proficiency", "Class_Tool_Expertise", "Class_Skill_Proficiency", "Class_Skill_Expertise", "Saving_Throw_Proficiency", "Class_Feat", "Background_Skill_Proficiency", "Background_Tool_Proficiency", "Background_Feat", "Background_Language"}
  local list = {"Inventory", "Equipped"}
  local i
  tPC = {}
  if love.filesystem.getInfo(location, "directory") then
    if love.filesystem.getInfo(location.."/Char.txt", "file") then tPC = readsingletable(location.."/Char.txt") end
    for i = 1, #table, 1 do
      if love.filesystem.getInfo(location.."/"..table[i]..".txt", "file") then
        tPC[table[i]] = readsingletable(location.."/"..table[i]..".txt")
      end
    end
    for i = 1, #list, 1 do
      if love.filesystem.getInfo(location.."/"..list[i]..".txt", "file") then
        tPC[list[i]] = readsinglelist(location.."/"..list[i]..".txt")
      end
    end
  end
end

function loadgame(location)  -- Load game by directory reference
  local i, name
  location = location or "Autosave"
  gGAMEMODE = "game"
  gPAGE, gPROFILE_PAGE, gSKILLS_CURSOR, gINV_CURSOR, gEQUIP_CURSOR = 0, 1, 1, 1, 1
  -- Load adventure details
  tADVENTURE, tMONTH, tHOLIDAY = {}, {}, {}
  if love.filesystem.getInfo(location.."/adventure.txt") then tADVENTURE = readsingletable(location.."/adventure.txt") end
  if love.filesystem.getInfo(location.."/calendar.txt") then tMONTH = readmultitable(location.."/calendar.txt")
  else tMONTH = readmultitable("data/calendar.txt") end
  if love.filesystem.getInfo(location.."/holidays.txt") then tHOLIDAY = readmultitable(location.."/holidays.txt")
  else tHOLIDAY = readmultitable("data/holidays.txt") end
  -- Load scenery in
  tSCENERY = {}
  if love.filesystem.getInfo(location.."/scenery.txt") then
    tSCENERY = readmultitable(location.."/scenery.txt")
  end
  -- Load areas in
  tAREA = {}
  if love.filesystem.getInfo(location.."/areas.txt") then
    tAREA = readmultitable(location.."/areas.txt")
  end
  -- Load character if there is one
  if love.filesystem.getInfo(location.."/Character", "directory") then
    tPC = {}
    name = love.filesystem.getDirectoryItems(location.."/Character")
    for i = 1, #name, 1 do
      if love.filesystem.getInfo(location.."/Character/"..name[i].."/Char.txt") then
        loadcharacter(location.."/Character/"..name[i])
      end
    end
  end
  -- Set PC location if not set yet
  if safetest(tPC.Location, 0) and next(tAREA)~=nil then
    for i = 1, #tAREA, 1 do
      if safetest(tAREA[i].Start, "Yes") and tAREA[i].ID~=nil then 
        tPC.Location = tAREA[i].ID
      end
    end
  end
  -- Load first area
  loadarea()
end

function loadarea()  -- Read in the text/music for this location
  local i, width
  stop_speech()
  gLINE, tAREA_TEXT, tSPEECH, gSPEECH_STRING, gSPEECH_LINE, gSPEECH_PLAYING = 1, {}, {}, "", 2, 0
  gAREA = getarea(tPC)
  gSCENE = getscene(gAREA)
  if gSCENE~=0 and tSCENERY[gSCENE].Music~=nil then musicplay(tSCENERY[gSCENE].Music) end
  if not safetest(tAREA[gAREA].Desc, nil) then width, tAREA_TEXT = fDEFAULT:getWrap(tAREA[gAREA].Desc, 200) end
  table.insert(tAREA_TEXT, 1, getdate().."\t"..gettime())
  table.insert(tAREA_TEXT, 2, " ")
  create_gamebuttons()
  erasefolder("cache")
  love.filesystem.createDirectory("cache")
end

function create_gamebuttons()   -- Main game screen buttons
  gSUBMENU = "main"
  tBUTTON = {}
  createbutton(tADVENTURE.Title, gCANVAS_W*0.5, 20, {font=fHEADING, width=200, g=215, b=0})
  createbutton(tAREA[gAREA].Name, gCANVAS_W*0.5, 44, {width=200, font=fSUBHEADING})
  createbutton("X", gCANVAS_W-30, 20, {image=iICON["small_off"], font=fSUBHEADING, width=22, clickable="Yes", alt_image=iICON["small_on"], script="setup_mainmenu"})
  createbutton(" ", 30, gCANVAS_H-20, {image=iICON["small_off"], font=fSUBHEADING, width=22, clickable="Yes", alt_image=iICON["small_on"], script="speech_onoff"})
  createbutton(" ", 30, gCANVAS_H-80, {image=iICON["small_off"], font=fSUBHEADING, width=22, clickable="Yes", alt_image=iICON["small_on"], script="loadarea"})
  createbutton(" ", gCANVAS_W-30, gCANVAS_H-20, {image=iICON["small_off"], font=fSUBHEADING, width=22, clickable="Yes", alt_image=iICON["small_on"], script="viewprofile"})
  if tAREA[gAREA].Continue~=nil then createbutton("Continue", gCANVAS_W*0.5, gCANVAS_H-50, {image=iICON["button_off"], font=fSUBHEADING, width=60, clickable="Yes", alt_image=iICON["button_on"], script="action_travel", var="Continue"}) end
  if tAREA[gAREA].North~=nil then createbutton("N", gCANVAS_W*0.5, gCANVAS_H-80, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="action_travel", var="North"}) end
  if tAREA[gAREA].South~=nil then createbutton("S", gCANVAS_W*0.5, gCANVAS_H-20, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="action_travel", var="South"}) end
  if tAREA[gAREA].East~=nil then createbutton("E", (gCANVAS_W*0.5)+50, gCANVAS_H-50, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="action_travel", var="East"}) end
  if tAREA[gAREA].West~=nil then createbutton("W", (gCANVAS_W*0.5)-50, gCANVAS_H-50, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="action_travel", var="West"}) end
  if gORIENTATION=="portrait" then
    if gettablesize(tAREA_TEXT)>15 then
      createbutton(" ", (gCANVAS_W*0.5)+110, 275, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_text", var="down"})
      createbutton(" ", (gCANVAS_W*0.5)+110, 65, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_text", var="up"})
    end
  else
    if gettablesize(tAREA_TEXT)>10 then
      createbutton(" ", (gCANVAS_W*0.5)+110, 200, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_text", var="down"})
      createbutton(" ", (gCANVAS_W*0.5)+110, 65, {image=iICON["small_off"], font=fSUBHEADING, width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_text", var="up"})
    end
  end
end

function speech_onoff()  -- toggle speech on/off
  if safetest(tSETTINGS.Speech, "No") then tSETTINGS.Speech = "Yes"
  else 
    tSETTINGS.Speech = "No"
    stop_speech()
  end
end

function stop_speech()  -- stop speech
  local i
  if tSPEECH~=nil and next(tSPEECH)~=nil then
    for i = 1, #tSPEECH, 1 do
      tSPEECH[i]:stop()
    end
  end
end

function scroll_text(direction)  -- Scroll area text up/down
  local max_lines
  if gORIENTATION=="portrait" then max_lines = 15
  else max_lines = 10 end
  if direction=="down" then
    gLINE = math.max(1, math.min(gettablesize(tAREA_TEXT)-(max_lines-1), gLINE+1))
  elseif direction=="up" then
    gLINE = math.max(1, gLINE-1)
  else end
end

function action_travel(direction)  -- Travel N/E/S/W/Continue etc.
  if not safetest(tAREA[gAREA][direction], nil) then 
    tPC.Location = tAREA[gAREA][direction]
  end
  loadarea()
end

function getdate()  -- returns the adventure date as a string
  return tADVENTURE.Day.." "..tMONTH[tADVENTURE.Month].Name..", "..tADVENTURE.Year
end

function gettime()  -- returns the adventure time as a string
  local h, m
  if tADVENTURE.Hour<10 then h = "0"..tADVENTURE.Hour
  else h = tADVENTURE.Hour end
  if tADVENTURE.Minute<10 then m = "0"..tADVENTURE.Minute
  else m = tADVENTURE.Minute end
  return h..":"..m
end

function savegame(location)  -- Save game at location
  location = location or "Autosave"
  if love.filesystem.getInfo(location, "directory") then erasefolder(location) end
  love.filesystem.createDirectory(location)
  writesingletable(location.."/adventure.txt", tADVENTURE)
  love.filesystem.createDirectory(location.."/Character")
  savecharacter(location.."/Character/"..tPC.Name)
  writemultitable(location.."/scenery.txt", tSCENERY)
  writemultitable(location.."/areas.txt", tAREA)
end

function accept_name()  -- save character, start game!
  if gTYPING==true then tPC.Name = gTYPEDTEXT end
  setnottyping()
  tPC.Location = 0
  tPC.Wounds = 0
  tPC.TempHP = 0
  tPC.Inspired = "No"
  tPC.Prof_Bonus = 2
  if string.len(tPC.HitDice)==4 then tPC.MaxHP = tonumber(string.sub(tPC.HitDice, 3, 4))+getmodifier(getabilityscore(tPC, "Constitution"))
  else tPC.MaxHP = tonumber(string.sub(tPC.HitDice, 3, 3))+getmodifier(getabilityscore(tPC, "Constitution")) end
  savecharacter("Characters/"..tPC.Name)
  loadgame("Adventures/"..tADVENTURE.Title)
  savegame()
end

function getarea(creature)  -- get the area number of the creature by location ID
  local area = 0
  local i
  if creature.Location~=nil and next(tAREA)~=nil then
    for i = 1, #tAREA, 1 do
      if safetest(tAREA[i].ID, creature.Location) then area = i end
    end
  end
  return area
end

function getscene(area)   -- get the scenery number of the area by name
  local scene = 0
  local i
  if area~=0 and tAREA[area].Scenery~=nil and next(tSCENERY)~=nil then
    for i = 1, #tSCENERY, 1 do
      if safetest(tAREA[area].Scenery, tSCENERY[i].Name) then scene = i end
    end
  end
  return scene
end

function viewprofile()   -- View character profile
  gSUBMENU = "profile"
  tINV_SELECTED = {}
  create_profilebuttons()
end

function create_profilebuttons()   -- Character profile buttons
  tBUTTON = {}
  createbutton(tPC.Name, gCANVAS_W*0.5, 20, {font=fHEADING, width=200, g=215, b=0})
  createbutton(tPC.Race.." "..tPC.Class..", Level "..tPC.Level, gCANVAS_W*0.5, 44, {width=200, font=fSUBHEADING})
  createbutton("X", gCANVAS_W-30, 20, {image=iICON["small_off"], font=fSUBHEADING, width=22, clickable="Yes", alt_image=iICON["small_on"], script="create_gamebuttons"})
  if gPROFILE_PAGE==1 then create_profilebasic()
  elseif gPROFILE_PAGE==2 then create_profileskills()
  elseif gPROFILE_PAGE==3 then create_profilebackpack()
  elseif gPROFILE_PAGE==4 then create_profileequipped()
  elseif gPROFILE_PAGE==5 then create_profilecombat()
  elseif gPROFILE_PAGE==6 then create_profilespells()
  elseif gPROFILE_PAGE==7 then create_profilefeats()
  elseif gPROFILE_PAGE==8 then create_profilemisc()
  else end
  create_profilefooter()
end

function create_profilebasic()   -- Display basic profile (abilities, XP, level etc.)
  local ability = {"Strength", "Dexterity", "Constitution", "Intelligence", "Wisdom", "Charisma"}
  local i, s, m
  for i = 1, #ability, 1 do
    s = getabilityscore(tPC, ability[i])
    m = getmodifier(s)
    if isproficient(tPC, ability[i]) then createbutton("X", (gCANVAS_W*0.5)-110, (i*24)+48, {image=iICON["small_on"], width=17, font=fSUBHEADING})
    else createbutton(" ", (gCANVAS_W*0.5)-110, (i*24)+48, {image=iICON["small_off"], width=17, font=fSUBHEADING}) end
    createbutton(string.upper(string.sub(ability[i], 1, 3)), (gCANVAS_W*0.5)-82, (i*24)+48, {image=iICON["small_off"], width=36, font=fSUBHEADING})
    createbutton(getabilityscore(tPC, ability[i]), (gCANVAS_W*0.5)-50, (i*24)+48, {image=iICON["small_off"], width=24, font=fSUBHEADING, g=215, b=0})
    if m>=0 then createbutton("(+"..m..")", (gCANVAS_W*0.5)-20, (i*24)+48, {image=iICON["small_off"], width=32, font=fSUBHEADING})
    else createbutton("("..m..")", (gCANVAS_W*0.5)-20, (i*24)+48, {image=iICON["small_off"], width=32, font=fSUBHEADING}) end
  end  
  createbutton(" HP: "..gethealth(tPC).."/"..tPC.MaxHP, (gCANVAS_W*0.5)+52, 72, {image=iICON["button_off"], font=fSUBHEADING, width=96, align="left"})
  createbutton(" AC: "..getarmorclass(tPC), (gCANVAS_W*0.5)+52, 96, {image=iICON["button_off"], font=fSUBHEADING, width=96, align="left"})
  createbutton(" XP: "..tPC.XP, (gCANVAS_W*0.5)+52, 120, {image=iICON["button_off"], font=fSUBHEADING, width=96, align="left"})
  createbutton(" Prof. Bonus: +"..tPC.Prof_Bonus, (gCANVAS_W*0.5)+52, 144, {image=iICON["button_off"], font=fSUBHEADING, width=96, align="left"})
  if getinitiative(tPC)>=0 then createbutton(" Initiative: +"..getinitiative(tPC), (gCANVAS_W*0.5)+52, 168, {image=iICON["button_off"], font=fSUBHEADING, width=96, align="left"})
  else createbutton(" Initiative: "..getinitiative(tPC), (gCANVAS_W*0.5)+52, 168, {image=iICON["button_off"], font=fSUBHEADING, width=96, align="left"}) end
  if safetest(tPC.Inspired, "Yes") then createbutton("Inspired", (gCANVAS_W*0.5)+52, 192, {image=iICON["button_on"], font=fSUBHEADING, width=96})
  else createbutton("Not Inspired", (gCANVAS_W*0.5)+52, 192, {image=iICON["button_off"], font=fSUBHEADING, width=96, t=80}) end
end

function create_profileskills()   -- Display skills/proficiencies
  local skill = readmultitable("data/skills.txt")
  local y = 66
  local i, mod, max
  if next(skill)~=nil then
    if gORIENTATION=="portrait" then max=9
    else max=7 end
    for i = gSKILLS_CURSOR, gSKILLS_CURSOR+max, 1 do
      mod = getskillmodifier(tPC, skill[i].Name)
      if mod>=0 then mod = "+"..mod end
      if isproficient(tPC, skill[i].Name) then 
        createbutton(mod, (gCANVAS_W*0.5)-91, y, {image=iICON["small_off"], font=fDEFAULT, width=17, g=215, b=0})
        createbutton(" "..skill[i].Name, (gCANVAS_W*0.5), y, {image=iICON["button_off"], font=fDEFAULT, width=164, g=215, b=0, align="left"})
      else 
        createbutton(mod, (gCANVAS_W*0.5)-91, y, {image=iICON["small_off"],font=fDEFAULT, width=17})
        createbutton(" "..skill[i].Name, (gCANVAS_W*0.5), y, {image=iICON["button_off"], font=fDEFAULT, width=164, align="left"})
      end
      y = y + 18
    end
  end
  createbutton(" ", (gCANVAS_W*0.5)+91, 66, {image=iICON["small_off"], width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_skills", var="up"})
  createbutton(" ", (gCANVAS_W*0.5)+91, 192, {image=iICON["small_off"], width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_skills", var="down"})
end

function scroll_skills(direction)  -- Scroll skills up/down
  local skill = readmultitable("data/skills.txt")
  local max
  if gORIENTATION=="portrait" then max=9 else max=7 end
  if direction=="up" then gSKILLS_CURSOR = math.max(gSKILLS_CURSOR-1, 1)
  elseif direction=="down" then gSKILLS_CURSOR = math.min(gSKILLS_CURSOR+1, #skill-max)
  else end
  create_profilebuttons()
end

function create_profilebackpack()  -- Display inventory
  local backpack = getbackpackitems()
  local y = 84
  local i, max
  createbutton("Num", (gCANVAS_W*0.5)-85, 66, {image=iICON["small_off"], font=fSUBHEADING, width=30, g=215, b=0})
  createbutton(" Item", gCANVAS_W*0.5, 66, {image=iICON["button_off"], font=fSUBHEADING, width=138, align="left", g=215, b=0})
  createbutton("lbs.", (gCANVAS_W*0.5)+85, 66, {image=iICON["button_off"], font=fSUBHEADING, width=30, g=215, b=0})
  createbutton(" ", (gCANVAS_W*0.5)+111, 66, {image=iICON["small_off"], width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_backpack", var="up"})
  createbutton(" ", (gCANVAS_W*0.5)+111, 192, {image=iICON["small_off"], width=17, clickable="Yes", alt_image=iICON["small_on"], script="scroll_backpack", var="down"})
  if next(backpack)~=nil then
    if gORIENTATION=="portrait" then max=7 else max=5 end
    gINV_CURSOR = math.max(1, math.min(gINV_CURSOR, #backpack-max))
    for i = gINV_CURSOR, math.min(#backpack, gINV_CURSOR+max), 1 do
      createbutton(backpack[i].Num, (gCANVAS_W*0.5)-85, y, {image=iICON["small_off"], font=fDEFAULT, width=30})
      if safetest(tINV_SELECTED[i], "Yes") then
        createbutton(" "..backpack[i].Name, gCANVAS_W*0.5, y, {image=iICON["button_on"], font=fDEFAULT, width=138, align="left", clickable="Yes", alt_image=iICON["button_on"], script="inv_deselect", var=i})
      else
        createbutton(" "..backpack[i].Name, gCANVAS_W*0.5, y, {image=iICON["button_off"], font=fDEFAULT, width=138, align="left", clickable="Yes", alt_image=iICON["button_on"], script="inv_select", var=i})
      end
      if backpack[i].Weight*backpack[i].Num==1 then
        createbutton("1lb.", (gCANVAS_W*0.5)+85, y, {image=iICON["small_off"], font=fDEFAULT, width=30})
      else
        createbutton(backpack[i].Weight*backpack[i].Num.."lbs.", (gCANVAS_W*0.5)+85, y, {image=iICON["small_off"], font=fDEFAULT, width=30})
      end
      y = y + 18
    end
  end
  if next(tINV_SELECTED)~=nil then createbutton("Drop", (gCANVAS_W*0.5)-80, y, {image=iICON["button_off"], font=fSUBHEADING, width=39, clickable="Yes", alt_image=iICON["button_on"], script="drop_items"})
  else createbutton("Drop", (gCANVAS_W*0.5)-80, y, {image=iICON["button_off"], font=fSUBHEADING, width=39, t=80}) end
  if isselectedequipable() then createbutton("Equip", (gCANVAS_W*0.5)-40, y, {image=iICON["button_off"], font=fSUBHEADING, width=39, clickable="Yes", alt_image=iICON["button_on"], script="equip_items"})
  else createbutton("Equip", (gCANVAS_W*0.5)-40, y, {image=iICON["button_off"], font=fSUBHEADING, width=39, t=80}) end
  if isselectedusable() then createbutton("Use", gCANVAS_W*0.5, y, {image=iICON["button_off"], font=fSUBHEADING, width=39, clickable="Yes", alt_image=iICON["button_on"], script="use_items"})
  else createbutton("Use", gCANVAS_W*0.5, y, {image=iICON["button_off"], font=fSUBHEADING, width=39, t=80}) end
  createbutton(getcarriedweight().."/"..(getabilityscore(tPC, "Strength")*15).."lbs.", (gCANVAS_W*0.5)+60, y, {image=iICON["button_off"], font=fSUBHEADING, width=80, g=215, b=0})
end

function isselectedequipable()   -- Can they equip the selected items?
  local backpack = getbackpackitems()
  local equip = false
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  if next(backpack)~=nil and next(tINV_SELECTED)~=nil then
    equip = true
    for i = 1, #backpack, 1 do
      if safetest(tINV_SELECTED[i], "Yes") then
        n = getitem(backpack[i].Name)
        if n~=0 then
          if item[n].Equip_Slot==nil then equip = false end
        end
      end
    end
  end
  return equip
end

function isselectedusable()   -- Can they use the selected item(s)?
  local backpack = getbackpackitems()
  local use = false
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  if next(backpack)~=nil and next(tINV_SELECTED)~=nil then
    use = true
    for i = 1, #backpack, 1 do
      if safetest(tINV_SELECTED[i], "Yes") then
        n = getitem(backpack[i].Name)
        if n~=0 then
          if item[n].Use==nil then use = false end
        end
      end
    end
  end
  return use
end

function inv_select(num)  -- Select this item in their backpack
  tINV_SELECTED[num] = "Yes"
  create_profilebuttons()
end

function inv_deselect(num)   -- Deselect this item in their backpack
  tINV_SELECTED[num] = "No"
  isinventoryselectempty()
  create_profilebuttons()
end

function isinventoryselectempty()    -- Check if tINV_SELECTED is empty and delete it if it is!
  local found = false
  for i,v in pairs(tINV_SELECTED) do
    if v=="Yes" then found = true end
  end
  if found==false then tINV_SELECTED = {} end
end

function scroll_backpack(direction)  -- Scroll skills up/down
  local backpack = getbackpackitems()
  local max
  if gORIENTATION=="portrait" then max=7 else max=5 end
  if direction=="up" then gINV_CURSOR = math.max(gINV_CURSOR-1, 1)
  elseif direction=="down" then gINV_CURSOR = math.min(gINV_CURSOR+1, #backpack-max)
  else end
  create_profilebuttons()
end

function getbackpackitems()  -- Returns an alphabetical table of backpack items with variable Num
  local list, item = {}, {}
  local water_weight, water_num = 0, 0
  local i, n
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt")
  else item = readmultitable("data/item_blueprints.txt") end
  if next(tPC.Inventory)~=nil then
    table.sort(tPC.Inventory)
    for i = 1, #tPC.Inventory, 1 do
      if next(list)~=nil and list[#list].Name==tPC.Inventory[i] then list[#list].Num = list[#list].Num + 1
      else
        n = getitem(tPC.Inventory[i])
        if n~=0 then
          if not safetest(item[n].Hidden, "Yes") then 
            table.insert(list, item[n])
            list[#list].Num = 1
          elseif safetest(item[n].Name, "Water") then
            water_weight = water_weight + item[n].Weight
            water_num = water_num + 1
          else end
        end
      end
    end
    if water_weight > 0 and next(list)~=nil then
      for i = 1, #list, 1 do
        if list[i].Name=="Waterskin" then
          if water_num > 5 then 
            list[i].Weight = list[i].Weight + (5*(water_weight/water_num))
            water_weight = water_weight - (5*(water_weight/water_num))
            water_num = water_num - 5
          elseif water_num > 0 then
            list[i].Weight = list[i].Weight + (water_num*(water_weight/water_num))
            water_weight, water_num = 0, 0
          else end
        end
      end
    end
  end
  return list
end

function getcarriedweight()   -- Returns the carried weight total (inc. hidden items)
  local total = 0
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if next(tPC.Inventory)~=nil then
    for i = 1, #tPC.Inventory, 1 do
      n = getitem(tPC.Inventory[i])
      if n~=0 then total = total + item[n].Weight end
    end
  end
  if next(tPC.Equipped)~=nil then
    for i = 1, #tPC.Equipped, 1 do
      n = getitem(tPC.Equipped[i])
      if n~=0 then total = total + item[n].Weight end
    end
  end
  return total
end

function equip_items()    -- Equip the selected items
  local backpack = getbackpackitems()
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if next(backpack)~=nil and next(tINV_SELECTED)~=nil then
    for i = 1, #backpack, 1 do
      if safetest(tINV_SELECTED[i], "Yes") then equip_item(backpack[i].Name) end
    end
  end
  tidy_equipslots()
  tINV_SELECTED = {}
  create_profilebuttons()
end

function equip_item(name)     -- Equip this item
  local equipped = false
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if next(tPC.Inventory)~=nil then
    for i = #tPC.Inventory, 1, -1 do
      n = getitem(tPC.Inventory[i])
      if n~=0 and tPC.Inventory[i]==name and equipped==false then
        if item[n].Equip_Slot~="Quiver" then equipped = true end
        table.insert(tPC.Equipped, 1, tPC.Inventory[i])
        table.remove(tPC.Inventory, i)
      end
    end
  end
end

function tidy_equipslots()    -- Put items back in inventory if too many in one slot
  local slot = readmultitable("data/equip_slots.txt")
  local count = {}
  local item, i, j, n, m
  local dual_wield_feat = hasfeat(tPC, "Dual Wielder")
  local main_hand, off_hand = getmainweapon(), getoffweapon()
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  -- Only allow dual wield if weapons are both light or player has dual wield feat, otherwise reduce weapons to one
  if off_hand~="None" then
    n = getitem(off_hand)
    m = getitem(main_hand)
    if n~=0 and m~=0 then
      if (not safetest(item[n].Light, "Yes") or not safetest(item[m].Light, "Yes")) and not safetest(item[m].Two_Handed, "Yes") and not hasfeat(tPC, "Dual Wielder") then
        for j = 1, #slot, 1 do
          if slot[j].Name=="Hand" then slot[j].Max=1 end
        end
      end
    end
  end
  -- Now zero all the counts for every slot
  if next(slot)~=nil then
    for j = 1, #slot, 1 do
      count[j]=0
    end
    -- Count how many items in each slot
    if next(tPC.Equipped)~=nil then
      for i = 1, #tPC.Equipped, 1 do
        n = getitem(tPC.Equipped[i])
        if n~=0 then
          for j = 1, #slot, 1 do
            if string.find(tPC.Equipped[i], "Quiver") and slot[j].Name=="Quiver" and slot[j].Alt~=nil then slot[j].Max=slot[j].Alt end  -- Quiver=20
            if item[n].Equip_Slot==slot[j].Name then
              count[j] = count[j] + 1
              if safetest(item[n].Two_Handed, "Yes") then count[j] = count[j] + 1 end   -- Two-handed weapons count double!
            end
          end
        end
      end
      -- Loop backwards and remove items until the count is within the limit!
      for i = #tPC.Equipped, 1, -1 do
        n = getitem(tPC.Equipped[i])
        if n~=0 then
          for j = 1, #slot, 1 do
            if item[n].Equip_Slot==slot[j].Name and count[j]>slot[j].Max then
              table.insert(tPC.Inventory, tPC.Equipped[i])
              table.remove(tPC.Equipped, i)
              count[j] = count[j] - 1
              if safetest(item[n].Two_Handed, "Yes") then count[j] = count[j] - 1 end
            end
          end
        end
      end
    end
  end
end

function create_profileequipped()  -- Display equipped info
  
end

function create_profilecombat()  -- Display combat info
  local con = getmodifier(getabilityscore(tPC, "Constitution"))
  local main_hand, off_hand = getmainweapon(), getoffweapon()
  local main_damage, off_damage = getdamage(tPC)
  local att_bonus, damage, damage_type
  createbutton(" Main: "..main_hand, (gCANVAS_W*0.5)-50, 66, {image=iICON["button_on"], font=fDEFAULT, width=99, align="left"})
  att_bonus = getattackbonus(tPC, main_hand)
  if att_bonus~=nil then
    if att_bonus>=0 then createbutton(" Att. Bonus: +"..att_bonus, (gCANVAS_W*0.5)-50, 84, {image=iICON["button_off"], font=fDEFAULT, width=99, align="left"}) else createbutton(" Att. Bonus: "..att_bonus, (gCANVAS_W*0.5)-50, 84, {image=iICON["button_off"], font=fDEFAULT, width=99, align="left"}) end
  end
  if next(main_damage)~=nil then
    createbutton(" Damage: "..main_damage.Damage.." ("..string.sub(main_damage.DamageType, 1, 1)..")", (gCANVAS_W*0.5)-50, 102, {image=iICON["button_off"], font=fDEFAULT, width=99, align="left"})
  end
  if off_hand~="None" then
    createbutton(" Off: "..off_hand, (gCANVAS_W*0.5)+50, 66, {image=iICON["button_on"], font=fDEFAULT, width=99, align="left"})
    createbutton(" Att. Bonus: ", (gCANVAS_W*0.5)+50, 84, {image=iICON["button_off"], font=fDEFAULT, width=99, align="left"})
    createbutton(" Damage: ", (gCANVAS_W*0.5)+50, 102, {image=iICON["button_off"], font=fDEFAULT, width=99, align="left"})
  end
  if con>=0 then createbutton(" Hit Dice: "..tPC.HitDice.."+"..con, (gCANVAS_W*0.5)-52, 144, {image=iICON["button_off"], font=fDEFAULT, width=96, align="left"}) else createbutton(" Hit Dice: "..tPC.HitDice..con, (gCANVAS_W*0.5)-52, 144, {image=iICON["button_off"], font=fDEFAULT, width=96, align="left"}) end
  createbutton(" Speed: "..getspeed(tPC).."ft.", (gCANVAS_W*0.5)-52, 168, {image=iICON["button_off"], font=fDEFAULT, width=96, align="left"})
  createbutton(" P. Perception: "..getpassiveperception(tPC), (gCANVAS_W*0.5)-52, 192, {image=iICON["button_off"], font=fDEFAULT, width=96, align="left"})
end

function getmainweapon()  -- Get weapon in main hand (or fist)
  local wep = "Fist"
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if next (tPC.Equipped)~=nil then
    for i = 1, #tPC.Equipped, 1 do
      n = getitem(tPC.Equipped[i])
      if n~=0 then
        if item[n].Equip_Slot=="Hand" and tPC.Equipped[i]~="Shield" and wep=="Fist" then wep=tPC.Equipped[i] end
      end
    end  
  end
  return wep
end  

function getoffweapon()  -- Get weapon name in off-hand (or none or shield)
  local main, wep = "None", "None"
  local i, n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if next(tPC.Equipped)~=nil then
    for i = 1, #tPC.Equipped, 1 do
      n = getitem(tPC.Equipped[i])
      if n~=0 then
        if item[n].Equip_Slot=="Hand" and tPC.Equipped[i]~="Shield" and main=="None" then main=tPC.Equipped[i]
        elseif item[n].Equip_Slot=="Hand" and wep=="None" then wep=tPC.Equipped[i]
        else end
      end
    end
  end
  return wep
end

function create_profilespells()  -- Display spellbook

end

function create_profilefeats()  -- Display feats

end

function create_profilefooter()   -- Profile menu buttons
  local menu = {"Basic", "Skills", "Backpack", "Equipped", "Combat", "Spells", "Feats", "Misc."}
  local x, y = (gCANVAS_W*0.5)-52, gCANVAS_H-80
  local i
  for i = 1, #menu, 1 do
    if gPROFILE_PAGE==i then createbutton(menu[i], x, y, {image=iICON["button_on"], font=fSUBHEADING, width=96})
    else createbutton(menu[i], x, y, {image=iICON["button_off"], font=fSUBHEADING, width=96, clickable="Yes", alt_image=iICON["button_on"], script="change_profile", var=i}) end
    y = y + 20
    if y>gCANVAS_H-20 then x, y = (gCANVAS_W*0.5)+52, gCANVAS_H-80 end
  end
end

function change_profile(page)  -- Choose a different profile view
  gPROFILE_PAGE = page
  create_profilebuttons()
end

function getabilityscore(creature, ability)   -- Get ability score
  local score = creature[string.sub(ability, 1, 3)]
  if creature[string.sub(ability,1,3).."_RacBonus"]~=nil then score = score + creature[string.sub(ability,1,3).."_RacBonus"] end
  return score
end

function getmodifier(score)   -- Convert score to modifier
  local value = {1, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30}
  local mod = {-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10}
  local m, i = 0, 1
  while score>=value[i] do
    m = mod[i]
    i = i + 1
  end
  return m
end

function gethealth(creature)   -- Get current HP
  local health = 0
  if creature.MaxHP~=nil then health = health + creature.MaxHP end
  if creature.TempHP~=nil then health = health + creature.TempHP end
  if creature.Wounds~=nil then health = health - creature.Wounds end
  return health
end

function getinitiative(creature)    -- Get Initiative
  local init = getmodifier(getabilityscore(creature, "Dexterity"))
  if hasfeat(creature, "Alert") then init = init + 5 end
  return init
end

function getarmorclass(creature)    -- Get armor class
  local ac, max_modifier = 10, math.huge
  local main_hand, off_hand = getmainweapon(), getoffweapon()
  local i, item, n, m
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if creature.Equipped~=nil and next(creature.Equipped)~=nil then
    for i = 1, #creature.Equipped, 1 do
      n = getitem(creature.Equipped[i])
      if n~=0 then
        if item[n].AC~=nil then ac = math.max(ac, item[n].AC) end
        if item[n].AC_Bonus~=nil then ac = ac + item[n].AC_Bonus end
        if item[n].MaxDexMod~=nil then max_modifier = math.min(max_modifier, item[n].MaxDexMod) end
      end
    end
  end
  if hasfeat(creature, "Dual Wielder") then
    n, m = getitem(main_hand), getitem(off_hand)
    if n~=0 and m~=0 and safetest(item[n].Melee, "Yes") and safetest(item[m].Melee, "Yes") then ac = ac + 1 end
  end
  ac = ac + math.min(getmodifier(getabilityscore(tPC, "Dexterity")), max_modifier)
  return ac
end

function getarmor(creature)   -- Get armor base type being worn
  local armor = "None"
  local item, i, n
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if creature.Equipped~=nil and next(creature.Equipped)~=nil then
    for i = 1, #creature.Equipped, 1 do
      n = getitem(creature.Equipped[i])
      if n~=0 and item[n].Equip_Slot~=nil and safetest(item[n].Equip_Slot, "Armor") then armor = item[n].Base end
    end
  end
  return armor
end

function getspeed(creature)   -- Get speed
  local speed = creature.Speed
  return speed
end

function getpassiveperception(creature)   -- Get Passive Perception
  local pass = 10
  pass = pass + getskillmodifier(creature, "Perception")
  return pass
end

function getitem(name)   -- Get item number by name
  local num = 0
  local i, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  if next(item)~=nil then
    for i = 1, #item, 1 do
      if safetest(item[i].Name, name) then num = i end
    end
  end
  return num
end

function getskillmodifier(creature, name)    -- Get skill modifier
  local skill = readmultitable("data/skills.txt")
  local n, score = 0, 0
  local i, ability
  for i = 1, #skill, 1 do
    if skill[i].Name==name then n = i end
  end
  if n~=0 then
    score = score + getmodifier(getabilityscore(tPC, skill[n].Ability))
  end
  if isproficient(creature, name) and creature.Prof_Bonus~=nil then score = score + creature.Prof_Bonus end
  return score
end

function getattackbonus(creature, weapon)  -- Get attack bonus
  local bonus = 0
  local n, item
  local str_mod, dex_mod = getmodifier(getabilityscore(creature, "Strength")), getmodifier(getabilityscore(creature, "Dexterity"))
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  n = getitem(weapon)
  if weapon=="Fist" and creature.Prof_Bonus~=nil then bonus = bonus + creature.Prof_Bonus + str_mod
  elseif n~=0 then
    if item[n].Base~=nil and isproficient(creature, item[n].Base) and creature.Prof_Bonus~=nil then bonus = bonus + creature.Prof_Bonus end
    if safetest(item[n].Finesse, "Yes") then
      if str_mod>dex_mod then bonus = bonus + str_mod
      else bonus = bonus + dex_mod end
    elseif safetest(item[n].Melee, "Yes") then bonus = bonus + str_mod
    elseif safetest(item[n].Ranged, "Yes") then 
      bonus = bonus + dex_mod
      if hasfeat(creature, "Archery") then bonus = bonus + 2 end
    else end
    if item[n].Enchant_Bonus~=nil then bonus = bonus + item[n].Enchant_Bonus end
  else end
  return bonus
end

function getdamage(creature)  -- Get physical damage from weapon
  local main_hand, off_hand = getmainweapon(), getoffweapon()
  local str_mod, dex_mod = getmodifier(getabilityscore(creature, "Strength")), getmodifier(getabilityscore(creature, "Dexterity"))
  local main_damage, off_damage = {}, {}
  local n, item
  if love.filesystem.getInfo("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") then item = readmultitable("Adventures/"..tADVENTURE.Title.."/item_blueprints.txt") else item = readmultitable("data/item_blueprints.txt") end
  -- Calculate physical damage for main hand
  n = getitem(main_hand)
  if main_hand=="Fist" then   -- Unarmed strike, monks do more etc.
    if hasfeat(creature, "Martial Arts") and getarmor(creature)=="None" then 
      if creature.Level<=4 then main_damage = {Damage="1d4", DamageType="Bludgeoning"}
      elseif creature.Level<=10 then main_damage = {Damage="1d6", DamageType="Bludgeoning"}
      elseif creature.Level<=16 then main_damage = {Damage="1d8", DamageType="Bludgeoning"}
      else main_damage = {Damage="1d10", DamageType="Bludgeoning"} end
    elseif hasfeat(creature, "Unarmed Fighting") then
      if off_hand=="None" then main_damage = {Damage="1d8", DamageType="Bludgeoning"} else main_damage = {Damage="1d6", DamageType="Bludgeoning"} end
    elseif hasfeat(creature, "Tavern Brawler") then main_damage = {Damage="1d4", DamageType="Bludgeoning"}
    else main_damage = {Damage="1d1", DamageType="Bludgeoning"} end
    if hasfeat(creature, "Martial Arts") and getarmor(creature)=="None" then
      if str_mod>dex_mod then
        if str_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..str_mod
        else main_damage.Damage = main_damage.Damage..str_mod end
      else
        if dex_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..dex_mod
        else main_damage.Damage = main_damage.Damage..dex_mod end
      end
    else
      if str_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..str_mod
      else main_damage.Damage = main_damage.Damage..str_mod end
    end
  elseif n~=0 then
    main_damage.Damage, main_damage.DamageType = item[n].Damage, item[n].DamageType
    if item[n].Versatile~=nil and off_hand=="None" then main_damage.Damage = item[n].Versatile end
    if item[n].Damage_Bonus~=nil then str_mod, dex_mod = str_mod+item[n].Damage_Bonus, dex_mod+item[n].Damage_Bonus end
    if safetest(item[n].Finesse, "Yes") then
      if str_mod>dex_mod then
        if str_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..str_mod else main_damage.Damage = main_damage.Damage..str_mod end
      else
        if dex_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..dex_mod else main_damage.Damage = main_damage.Damage..dex_mod end
      end
    elseif safetest(item[n].Melee, "Yes") then
      if str_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..str_mod else main_damage.Damage = main_damage.Damage..str_mod end
    elseif safetest(item[n].Ranged, "Yes") then
      if dex_mod>=0 then main_damage.Damage = main_damage.Damage.."+"..dex_mod else main_damage.Damage = main_damage.Damage..dex_mod end
    else end
  else end
  return main_damage, off_damage
end